/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     defs.h
 * CONTENTS: main header file, included by all parts (*.c) except
 *	     version.c
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#define IntuitionBase_DECLARED
#define GfxBase_DECLARED
#define ReqToolsBase_DECLARED

/*
** Define only ONE of the following Constants. ALPHA, BETA, FD, REG
*/

#define ALPHA 1 // Alpha Release, Internal Use
/*
#define BETA  1 // Beta Release
#define FD    1 // Freely Distributable version
#define REG   1 // Registered Version
*/

#include "osstd.h"
#include "gadtools.h"
#include "std_ob.h"

#define OR  ||
#define AND &&

#define BOR  |
#define BAND &

#define CP_PCBMEM_SIZE	40000	/* Memory for PCB 40000 Bytes */
#define CP_PARTMEM_SIZE 20000	/* Memory for Part 20000 Bytes */
#define CP_FONTMEM_SIZE 20000	/* Memory for vector font */

#define CP_SELECTED    1
#define CP_UNSELECTED  2
#define CP_NOMODIFY   -1

#define CP_CLEAR	0
#define CP_INVERS      -1
#define CP_SELECT      -2
#define CP_NORM        -3
#define CP_NORMQUICK   -4
#define CP_INVERSQUICK -5

#define CP_COMPLETE  128

#define CP_LAYER_ACTIVE   1
#define CP_LAYER_INACTIVE 2
#define CP_LAYER_HIDDEN   3

#define CP_SOLDERLAYER	  1
#define CP_COMPONENTLAYER 2
#define CP_LABELLAYER	  3

#define CP_IMG_LABEL 1
#define CP_IMG_LAYER 2

#define CP_ADD_TRACE   1
#define CP_ADD_PAD     2
#define CP_ADD_LINE    3
#define CP_ADD_CIRCLE  4
#define CP_ADD_ELLIPSE 5
#define CP_ADD_RECT    6
#define CP_ADD_TEXT    7
#define CP_ADD_BEZIER  8    // for Bezier-Splines (unused)
#define CP_ADD_CONNECT 9    // for unrouted Connections (unused)
#define CP_ADD_DRAWING 10   // for Symbols (Labels) (unused)

#define CP_PART_COPY   11
#define CP_PART_CUT    12
#define CP_PART_CLEAR  13
#define CP_PART_ADD    14

// If compiled with the 2.1 Includes. Remove if compiled with 3.0
#define SafeSetWriteMask(rp,mask) SetWrMsk(rp,mask)


union elements
{
    struct Element  *element;
    struct Trace    *trace;
    struct Pad	    *pad;
    struct Text     *text;
    struct Line     *line;
    struct Ellipse  *ellipse;
    struct Rect     *rect;
    struct Circle   *circle;
};

struct Element
{
    UBYTE Type,Item;
};

/*
** struct Pad
**
** should be replaced be struct NewPad
*/

struct Pad
{
    UBYTE Type,Size;	    // Type = CP_PAD_ID, Number of Padsize
    SHORT x_pos,y_pos;	    // Well, what could that be... ??
};

#define CP_PADLAYER_SOLDER     1    // Completly unused
#define CP_PADLAYER_COMPONENT  2    // Useful for struct NewPad
#define CP_PADLAYER_ALL        3    //	  "

#define CP_PADTYPE_RECT        1    //	  "
#define CP_PADTYPE_ELLIPSE     2
#define CP_PADTYPE_EIGHT       3    //	  "

/*
** NewPad (unused)
**
** Connections are only possible between PADs !
*/

struct NewPad
{
    UBYTE  Type, Fill;	    // PAD_ID, Unused
    char   Id[4];	    // PadID, f.e. "1".."99" or "A1" (PGAs), not NULL terminated
    char   Name[8];	    // Padname, f.e. "_CLK", not NULL terminated, max. 8 chars
    UBYTE  Layer;	    // CP_PADLAYER_ALL | CP_PADLAYER_SOLDER | CP_PADLAYER_COMPONENT
    UBYTE  PadType;	    // CP_PADTYPE_RECT | CP_PADTYPE_ELLIPSE | CP_PADTYPE_EIGHT
    USHORT SizeH, SizeV;    // SizeH = SizeV : PAD is a circle
    USHORT SizeHole;	    // SizeHole = 0 : No Hole
    SHORT  PosX, PosY;
};

/*
** TraceEntry
**
** Useful for Traces and Connections beween PADs.
** (OK, let's say you have two PADs and you want to make a connection (aka
** a trace) between)
*/

#define CP_CONNECT_TRUE 	   // Es existiert eine Leiterbahn
#define CP_CONNECT_FALSE	   // Es existiert KEINE Leiterbahn
#define CP_CONNECT_IGNORE	   // Es existiert eine Leiterbahn, die aber keine
				   // Pads verbindet.
struct xy
{
    SHORT x, y;
};

struct TraceEntry
{
    UBYTE  Type,IsConnect;	// Wenn IsConnect = FALSE, dann sind alle folgenden
				// Eintrage, bis auf die PadIDs, NULL.
    UBYTE  Layer,Fill;
    char   StrtPrtID[4],StrtPadID[4];	// Wenn PrtID=NULL, dann ist es kein Bauteil, sondern
    char   EndPrtID[4],EndPadID[4];	// direkt ein Pad. Durch die 4 Byte ist ein schneller
					// Langwort Vergleich moeglich.
    SHORT  x_start,y_start;	// Startkoordinaten der Leiterbahn
    SHORT  x_end,y_end; 	// Endkoordinaten der Leiterbahn
    USHORT Count;		// Anzahl der Koordinaten, Jede Koordinate ist eine
				// Ecke im Leiterbahnzug.
    USHORT Size;		// Dicke der Leiterbahn
/*
** Now the XY coords start.
**
** struct xy	*ptrxy;
** struct TraceEntry *entry;
**
** ptrxy = (char *) entry + sizeof (TraceEntry);
** nextptr = ++ptrxy;
** Ptr to Next Element: nxt = entry + sizeof(TraceEntry) + (entry->Count * sizeof(struct xy);
*/
};

/*
** struct Trace
**
** Beinhaltet eine gerade,durchgezogene Leiterbahn der Dicke Thickness
*/

struct Trace
{
    UBYTE Type,Layer;
    USHORT Size;	    // Breite der Leiterbahn in 1/1600 Inch
    SHORT x_start,y_start;
    SHORT x_end,y_end;
};

/*
** struct Text
**
** Zum schreiben von Texten im Bestckungsaufdruck oder den Layern.
*/

struct Text
{
    UBYTE Type, Layer;
    UBYTE Size, Angle;	       /* FontGre, Winkel in Grad */
    SHORT Textsize;	       /* Anzahl der Text-Bytes */
    SHORT x_pos,y_pos;
/*
**  char "text..."      // Es folgt gleich darauf der Text
**			// Ist die Anzahl ungerade (z.B. "text" laenge = 5)
**			// Mu noch ein zustzliches 0 FllByte eingefgt werden
**			// Text.TextSize mu immer gerade sein
**			// daraus ergibt sich:
**  laenge = sizeof(struct Text) + Text.Textsize;
*/
};

#define CP_PART_STD  1	// Std-Bauteil, z.B. Widerstand, Elko, IC in DIL ausfuehrung
#define CP_PART_DIL  2	// DIL IC
#define CP_PART_SMD  3	// SMD Bauteil
#define CP_PART_PGA  4	// Pin Grid Array IC
#define CP_PART_PLCC 5	// Plastic Leadless Chip Carrier IC
#define CP_PART_LCC  6
#define CP_PART_CC   7

/*
** struct Part
**
** Beinhaltet eine komplettes Bauelement. Sie koennen im Layoutmodus NICHT
** modifiziert werden, dazu ist alleine der Bauteileeditor zustaendig.
** Veraendert werden koennen: IDStr, Name, Position und die Namen der Pads.
** Der Winkel des Bauteils (0,90,...) kann auch veraendert werden, aber
** nur indem die Positionen aller Eintrage gedreht werden.
** Die IDs der Pads sind unveraenderbar. Ein Bauteil wird immer komplett
** aus einer Library in die Platine kopiert, d.h. ein Bauteil hat, wenn es
** einemal eigesetzt ist, keine Bezuege mehr zu seinem Ursprung.
*/

struct Part
{
    UBYTE   Type, Kind; 	    // CP_PART_ID, Art des Bauteiles z.b. CP_SMD_ID
    char    ID[4];		    // ID-Str: z.B. R1, U21, C4, L9 usw. (wird
				    // angezeigt!) Doppeltbenennung NICHT moeglich
				    // 4 Zeichen OHNE! NullByte
    char    Name[12];		    // Name: z.B. MC68000, 74LS124 usw.
				    // Doppelnennung moeglich
				    // 12 Zeichen OHNE! Nullbyte
    SHORT   RefPosX, RefPosY;	    // Referenzposition; zu dieser Position
				    // muessen die Koordinaten der einzelnen
				    // Elemente dazu addiert werden
    SHORT   SizeX, SizeY;	    // Gre des Bauteils in 1/1600 in
				    // RefPos und Size definieren gleichzeitig
				    // ein Rechteck, mit dem das Bauteil selektiert
				    // werden kann.
    USHORT  Size;		    // Anzahl der Bytes fuer alle Elemente
    USHORT  Elements;		    // Anzahl der noch folgenden Elemente
    SHORT   IdPosX, IdPosY;	    // Position des ID-Textes
    UBYTE   Angle, Fill;	    // Winkel
};

/*
** Nun folgen die eigentlichen Elemente
** die Lanege des Parts berechnet sich wie folgt:
** laenge = sizeof(struct Part) + Part.length;
*/

/*
** struct Line
**
** Beinhaltet eine durchgezogene Line der Dicke Thickness
** fuer den Bestckungsaufdruck
*/

struct Line
{
    UBYTE Type,Item;
    USHORT Thickness;
    SHORT x_start,y_start;
    SHORT x_end,y_end;
};


/*
** struct Ellipse
**
** Ellipse fuer den Bestckungsaufdruck. Auch fuer Kreise
*/

struct Ellipse
{
    UBYTE Type,Item;	    // CP_LABEL_ID, CP_ELLIPSE_ID
    USHORT Thickness;	    // in 1/1600 in
    SHORT x_pos,y_pos;
    SHORT size_x,size_y;    // size_x == size_y dann Kreis :-)
};

/*
** struct Bezier
**
** Bezier-Kurfe fuer den Bestckungsaufdruck
*/

struct Bezier
{
    UBYTE Type,Item;
    USHORT Thickness;
    SHORT x_start, y_start;
    SHORT x_end,   y_end;
    SHORT x_ref,   y_ref;
};

/*
** struct Rect
**
** Rechteck fuer den Bestckungsaufdruck
*/

struct Rect
{
    UBYTE Type,Item;
    USHORT Thickness;
    SHORT x_pos,y_pos;
    SHORT x1_pos,y1_pos;
};


/* Aufbau eines CrazyPads File:
** name.ext
** ext = .cplay Schaltplan File (Schaltplaneditor noch nicht fertig)
**	 .cppcb PCB-Layout File
**	 .cpall PCB-All-Layout File
**	 .cpprt Bauteil File
**	 .cplib Bauteile Bibliothek
** Ausnahme:
**	 .prefs        (Defaultfile fuer Crazypads, befindet sich
**			in ENVARC:)
**
** CrazyPads versucht zum Programmstart, in folgender Reihenfolge ein
** Config-File zu oeffnen:
**
** ENVARC:CrazyPads/Default.prefs
** ENVARC:CrazyPads.prefs
**
** Das Standard Verzeichnis fuer ALLE Config-Files ist
** ENVARC:CrazyPads/
**
** File Header:
*/

struct CP_FileHeader
{
    ULONG   Head;	// CrazyPads Kennung; Head = 'CZPD'
    ULONG   Length;	// Lnge der nach dem FileHeader folgenden Daten
    ULONG   Version;	// CrazyPads Version, mit der das File erstellt wurde
    ULONG   ChkSum;	// Checksumme der nach dem FileHeader folgenden Daten
};

struct CP_Chunk
{
    ULONG   Head;	// Kennung, z.B. 'CZPD'
    ULONG   Length;	// Lnge des Chunks in _Bytes_
};			// Nchster Chunk: &Chunk + sizeof (CP_Chunk) + Length;
			// Length = DataLen + sizeof (HeadChunk) - sizeof (CP_Chunk)

/* Definierte Chunks:
**
**  CZPD    HeaderChunk fr CrazyPads
**  CFG0    Konfigdatei   (Version 0)
**  PCB0    Layout Datei  (Version 0)
**  LIB0    Library datei (Version 0)
**  PRT0    Parts datei   (    "    )
**  SCM0    Scheme Datei
*/

struct CP_Layout
{
    ULONG   Head;
    ULONG   Length;
    ULONG   Baseresolution;
};

#define CP_GREY_4	1
#define CP_GREY_8	2
#define CP_COLOR_4	3
#define CP_COLOR_8	4
#define CP_COLOR_16	5
#define CP_COLOR_256	6
#define CP_COLOR_CUSTOM 7

#define CP_COL_SELECT	    0
#define CP_COL_CLS	    1
#define CP_COL_COMP	    2
#define CP_COL_COMPINACT    3
#define CP_COL_SOLDER	    4
#define CP_COL_SOLDERINACT  5
#define CP_COL_LABEL	    6
#define CP_COL_LABELINACT   7
#define CP_COL_RASTER	    8
#define CP_COL_CONNECT	    9
#define CP_COL_PARTS	   10
#define CP_COL_PAD	   11
#define CP_COL_BACKGROUND  12
#define CP_COL_LIGHT	   13
#define CP_COL_DARK	   14

#define CP_DRAWSPEED_NORM      1
#define CP_DRAWSPEED_OUTLINE   2
#define CP_DRAWSPEED_FAST      3

#define CP_UNIT_MILLIMETER 1
#define CP_UNIT_INCH	   2

struct PadDef
{
    USHORT  a_size, b_size, hole_size;
    UBYTE   Type, Layer;
    UBYTE   Hole, Fill;
};

struct CP_Config
{
    ULONG   Head;	// Config Kennung; Head = 'CFG0'
    ULONG   Length;	// Lnge von des CFG0 Chunks
    ULONG   PrefsLen;	// Lnge der Pref-Strukt (muss NICHT immer identisch
			//			  mit Length sein!!!)
    ULONG   ViewMode;
    ULONG   OScanMode;
    USHORT  BitPlanes, Width, Height;
    USHORT  Color[16];	    // Color im Format 0x0000 bis 0x0fff
    USHORT  ColorMode;	    // Voreigestellte Farbenzuordnungen, z.B. CP_COLOR_8
    UBYTE   ColorDef[32];   // defcolentry = ColorDef[CP_COL_#?]
			    // Also: Farbcode = Color[ColorDef[CP_COL_LIGHT]]
    char    FontName[30];   // Name des Zeichensatzes,
    USHORT  FontSize;	    // Groesse
    USHORT  TraceSize[10],LabelSize[10];
    struct  PadDef PadEntry[10];
    USHORT  BaseScrDPI, Aspect;
    USHORT  ScrollStep;
    UBYTE   ScrollRel;
    UBYTE   Units;
    UBYTE   UseArea, UseScroll, UseBlitBitMap;
    UBYTE   DrawSpeedPads, DrawSpeedTraces, DrawSpeedLabel;
    UBYTE   Fill;
    char    PlotName[30];
    USHORT  PrtLines;
    USHORT  PrtDotSize;
    USHORT  BaseResolution; // Bestimmt die Genauigkeit u. die Gre der Platine
			    // Der Wertebereich geht von -32767 bis +32768
			    // Wenn nun Baseresolution = 1000 ist, dann entsprechen
			    // 1000 Einheiten einem Inch, also ist die maximale
			    // Platinen Gre 32768/1000 = 32.768 Inch oder 83.23cm
			    // Bei BasRes = 1600 sinds dann noch 52.02cm
    ULONG   reserved[10];   // Ein paar reservierte Sachen :-)
};

struct PCB
{
    struct Element *Start;
    struct Element *End;
    ULONG  Size;
    char   Name[30];
    char   Dir[30];
    USHORT BaseDpiX,BaseDpiY;	// in 1/1600 in
    USHORT ZoomFakt;		// in Prozent eg. 125 = 1.25
    short  ActStep;
    USHORT StepSize;		// Editraster (step)
    short  ActGrid;
    USHORT GridSize;		// Punkt Raster, beginnent bei (0,0)
    BOOL   GridFlag;
    BOOL   XYFlag;
    short  OffsetX, OffsetY;
    struct FileLock *DirLock;
};

extern struct PCB *ActualPCB;

#define PCBSTART (ActualPCB->Start)
#define PCBEND	 (ActualPCB->End)
#define PCBSIZE  (ActualPCB->Size)
#define PCBNAME  (&(ActualPCB->Name))
#define PCBDIR	 (&(ActualPCB->Dir))
#define PCBBASEX (ActualPCB->BaseDpiX)
#define PCBBASEY (ActualPCB->BaseDpiY)
#define PCBZOOM  (ActualPCB->ZoomFakt)
#define PCBACTSTEP (ActualPCB->ActStep)
#define PCBSTEP    (ActualPCB->StepSize)
#define PCBACTGRID (ActualPCB->ActGrid)
#define PCBGRID    (ActualPCB->GridSize)
#define PCBGRIDFLAG (ActualPCB->GridFlag)
#define PCBXYFLAG   (ActualPCB->XYFlag)
#define PCBOFFSETX  (ActualPCB->OffsetX)
#define PCBOFFSETY  (ActualPCB->OffsetY)
#define PCBLOCK     (ActualPCB->DirLock)

#define Zoll2X(xk) ((short) ((long) (xk - PCBOFFSETX)*ActualdpiX/1600))
#define Zoll2Y(yk) ((short) ((long) (yk - PCBOFFSETY)*ActualdpiY/1600+StatusHeight))
#define Zoll2PixX(xk) ((short) ((long) xk*ActualdpiX/1600))
#define Zoll2PixY(yk) ((short) ((long) yk*ActualdpiY/1600))

#define CP_PAD_ID	 1
#define CP_TRACE_ID	 2
#define CP_LABEL_ID	 3
#define CP_TEXT_ID	 4
#define CP_PART_ID	 5

#define CP_LINE_ID    1
#define CP_ELLIPSE_ID 2
#define CP_RECT_ID    3
#define CP_CIRCLE_ID  4
#define CP_BEZIER_ID  5

#define NULL	    0
#define Prototype   extern
#define Local	    static

