/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     dock.c
 * CONTENTS: Main parts of the user interface for CrazyPads
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "debug.h"
#include "/obj/prog-protos.h"

Prototype void DoOnXY(void);
Prototype void DoOffXY(void);
Prototype void OpenDock(void);
Prototype void CloseDock(void);
Prototype struct Gadget *GetGadAddr(struct Window *GadWin, UWORD GadID);
Prototype void CompNewDPI(void);
Prototype void ZoomPlus(void);
Prototype void ZoomMinus(void);
Prototype void SelectGadSt(void);
Prototype void SelectDock(void);
Prototype void SelectRawKey(void);
Prototype void SelectVanillaKey(void);
Prototype void InitGadLink(struct Gadget *prev, struct Gadget *buff,x,y,width,height,id);
Prototype void UpdateItem(short id, short select, short color);
Prototype void ChangeSize(short NewSize);
Prototype void UpdateColors(void);
Prototype void DoSorry(void);

Prototype void MoveLeft(void);
Prototype void MoveRight(void);
Prototype void MoveUp(void);
Prototype void MoveDown(void);

Prototype short  DockSizeX, SelItem;
Prototype short dockx, docky, fontheight;

Local	  void NewTool(short layer, short newtool);
Local	  void SelActive(int Layer);
Local	  void CheckIt(void);
Local	  void SelComponent(void);
Local	  void SelSolder(void);
Local	  void SelLabel(void);
Local	  void UpdateItemText(short id, char *txt);
Local	  int  DrawItem(short id);
Local	  void SelNewStepSize(void);
Local	  void SelPadLayer(void);

void NewTool(short layer, short newtool)
{
    CleanUp();

    if (layer == IMG_LABEL)
	SelActive(LABELLAYER);
    else if (layer == IMG_LAYER)
    {
	    if (ActiveLayer == LABELLAYER)
	        SelActive(SOLDERLAYER);
	    else
	        SelActive(ActiveLayer);
    }
    Tool = newtool;
}

void ChangeSize(short NewSize)
{
    if (Tool == ADD_PAD)
    {
	    DrawPad(x2,y2,PinSize,INVERS);
	    PinSize = NewSize;
	    DrawPad(x2,y2,PinSize,INVERS);
    }
    else if (Tool == ADD_TRACE)
    {
	    if (DrawKey)
	        Draw45Trace(INVERS);

	TraceSize = Cfg.TraceSize[NewSize];

	if (DrawKey)
	    Draw45Trace(INVERS);
    }
}

void CheckIt(void)
{
    struct Trace *MyTrace;
    struct Pad	 *MyPad;

    if (DelFlag == 1)
    {
	MyPad = FindPad(x1,y1);
	DrawPad(x1,y1,MyPad->Size,INVERS);    /* Altes, invert. Ltpin normal */
    }
    if (DelFlag == 3)
    {
	MyTrace = FindTraceLayer(x1,y1,x2,y2,ActiveLayer);
	DrawTrace(x1,y1,x2,y2,MyTrace->Size,INVERS);
    }
}

void DoOnXY(void)
{
    PCBXYFLAG = TRUE;
    kx = x;
    ky = y;

    if (HideArea)
	HideXYFlag = TRUE;
    else
    {
	DrawXY(kx,ky);
	HideXYFlag = FALSE;
    }
}

void DoOffXY(void)
{
    if (!HideXYFlag && PCBXYFLAG)
	DrawXY(kx,ky);

    PCBXYFLAG = FALSE;
    HideXYFlag = 0;
}

short dockx, docky, fontheight;

void Draw3dBox(short x, short y, short width, short height, short mode, short color)
{
    SetDrMd(RPd,JAM1);
    SetAPen(RPd,color);

    RectFill(RPd,x,y,x+width,y+height);

    if (mode == SELECTED)
    {
	SetAPen(RPd, 2);    // Weiss
	Move(RPd, x, y+height);
	Draw(RPd, x+width, y+height);
	Draw(RPd, x+width, y);

	SetAPen(RPd, 1);    // Schwarz
	Move(RPd, x, y+height);
	Draw(RPd, x, y);
	Draw(RPd, x+width, y);
    }
    else
    {
	SetAPen(RPd, 1);    // Schwarz
	Move(RPd, x, y+height);
	Draw(RPd, x+width, y+height);
	Draw(RPd, x+width, y);

	SetAPen(RPd, 2);    // Weiss
	Move(RPd, x, y+height);
	Draw(RPd, x, y);
	Draw(RPd, x+width, y);
    }
}

struct LayItem
{
    short Id;
    short pos_x, pos_y;
    char  text[20];
    char  select, color;
    ULONG exclude;
};

struct LayItem Items[] = {
     3, 1, 1,"_Add",  UNSELECTED, 0, 0,
     8, 2, 1,"_Cut",  UNSELECTED, 0, 0,
     5, 1, 2,"Cop_y", UNSELECTED, 0, 0,
     7, 2, 2,"_Edit", UNSELECTED, 0, 0,
    23, 1, 3,"_Lib",  UNSELECTED, 0, 0,
    24, 2, 3,"9_0",  UNSELECTED, 0, 0,
     4, 1, 4,"_Del",  UNSELECTED, 0, 0,
     1, 2, 4,"_Trace",UNSELECTED, 0, 0,
     2, 1, 5,"_Pad",  UNSELECTED, 0, 0,
     6, 2, 5,"Te_xt", UNSELECTED, 0, 0,
     9, 1, 6,"_Undo", UNSELECTED, 0, 0,
    10, 2, 6,"Li_ne", UNSELECTED, 0, 0,
    11, 1, 7,"_Rect", UNSELECTED, 0, 0,
    12, 2, 7,"Ell_ip",UNSELECTED, 0, 0,
    13, 1, 8,"La_bl", UNSELECTED, 0, 0,
    14, 2, 8,"Co_mp", UNSELECTED, 0, 0,
    15, 1, 9,"_Sold", SELECTED,   0, 0,
    16, 2, 9,"PadLy", UNSELECTED, 0, 0,
    17, 1,10,"Conct", UNSELECTED, 0, 0,
    18, 2,10,"Parts", UNSELECTED, 0 ,0,
    19, 3,11,"Zoom 1.0",        UNSELECTED,0, 0,
    20, 3,12,"_Grid 2.54 mm",   UNSELECTED,0, 0,
    21, 3,13,"Step 2.54 mm",    UNSELECTED,0, 0,
    22, 3,14,"_W:Arc 0",       UNSELECTED,0, 0,
//    25, 1,13,"Left",    UNSELECTED, 0 ,0,
//    26, 4,13,"Right",   UNSELECTED, 0 ,0,
//    27, 2,13,"Down",    UNSELECTED, 0 ,0,
//    28, 2,12,"Up",      UNSELECTED, 0 ,0,

     0, 0, 0, "", UNSELECTED, 0, 0

};

void UpdateItem(short id, short select, short color)
{
    short cnt;

    cnt = 0;
    do
    {
	if (Items[cnt].Id == 0)
	    return;
    }
    while (Items[cnt++].Id != id);

    cnt--;

    if (select != NOMODIFY)
	Items[cnt].select = select;
    if (color != NOMODIFY)
	Items[cnt].color = color;

    DrawItem(id);
}

void UpdateItemText(short id, char *txt)
{
    short cnt;

    cnt = 0;
    do
    {
	if (Items[cnt].Id == 0)
	    return;
    }
    while (Items[cnt++].Id != id);

    cnt--;

    strcpy(Items[cnt].text, txt);

    DrawItem(id);
}

short GetIdXY(short xp, short yp)
{
    short cnt = 0;

    FOREVER
    {
	if (Items[cnt].Id == 0)
	    return(FALSE);
	if ((Items[cnt].pos_x BAND xp) AND (Items[cnt].pos_y == yp))
	    return(Items[cnt].Id);
	cnt++;
    }
}

int DrawItem(short id)
{
    short len, cnt, udpos, udlen, start;
    short x,y;
    short pos_x, pos_y, select, color;
    char  txt[40];

    cnt = 0;
    do
    {
	if (Items[cnt].Id == 0)
	    return(FALSE);
    }
    while (Items[cnt++].Id != id);

    cnt--;

    strcpy(txt,Items[cnt].text);
    pos_x = Items[cnt].pos_x;
    pos_y = Items[cnt].pos_y;
    select = Items[cnt].select;
    color = Items[cnt].color;

    switch (pos_x)
    {
	case 1:
	    len = dockx/2-4;
	    x	= 4;
	    break;

	case 2:
	    len = dockx/2-4;
	    x	= DockSizeX/2+2;
	    break;

	case 3:
	    len = 2*dockx/2-4;
	    x	= 4;
	    break;

	case 4:
	    len = dockx/2-4;
	    x	= 2*DockSizeX/2;
	    break;

	case 6:
	    len = 2*dockx/3-4;
	    x	= DockSizeX/2+3;
	    break;

	case 7:
	    len = dockx-4;
	    x	= 4;
	    break;
    }

    y = (pos_y-1)*(fontheight+6) + 3;

    Draw3dBox(x,y,len,fontheight+4, select, color);

    udlen = 0;

    for (cnt=0; cnt<strlen(txt); cnt++)
    {
	    if (txt[cnt] == '_')
	    {
	        strcpy(&txt[cnt], &txt[cnt+1]);
	        udpos = TextLength(RPd, txt, cnt);
	        udlen = TextLength(RPd, txt, cnt+1);
	    }
    }

    if (color != 1)
	    SetAPen(RPd,1);
    else
	    SetAPen(RPd,2);

    start = x+(len-TextLength(RPd, txt, strlen(txt)))/2;

    Move(RPd,start,y+fontheight);
    Text(RPd,txt,strlen(txt));

    if (udlen>0)
    {
	Move(RPd,start+udpos, y+fontheight+2);
	Draw(RPd,start+udlen, y+fontheight+2);
    }
    return(TRUE);
}

short GetDockXPos(short x_pos)
{
    if (x_pos > DockSizeX/3)
    {
	if (x_pos > 2*DockSizeX/3)
	    return (4);
	else
	    return (2);
    }
    else
	return (1);
}

short GetDockYPos(short y_pos)
{
    return((y_pos-3)/(fontheight+6)+1);
}

void SelComponent(void)
{
    SetDrMd(RPd,JAM1);

    switch (ComponentLayer)
    {
	case LAYER_ACTIVE:
	    ComponentLayer = LAYER_INACTIVE;
	    UpdateItem(14,NOMODIFY,CompoInactiveCol);
	    break;

	case LAYER_INACTIVE:
	    ComponentLayer = LAYER_HIDDEN;
	    UpdateItem(14,NOMODIFY,ClsCol);
	    break;

	case LAYER_HIDDEN:
	    ComponentLayer = LAYER_ACTIVE;
	    UpdateItem(14,NOMODIFY,CompoLayerCol);
	    break;
    }
}

void SelSolder(void)
{
    SetDrMd(RPd,JAM1);

    switch (SolderLayer)
    {
	case LAYER_ACTIVE:
	    SolderLayer = LAYER_INACTIVE;
	    UpdateItem(15,NOMODIFY,SolderInactiveCol);
	    break;

	case LAYER_INACTIVE:
	    SolderLayer = LAYER_HIDDEN;
	    UpdateItem(15,NOMODIFY,ClsCol);
	    break;

	case LAYER_HIDDEN:
	    SolderLayer = LAYER_ACTIVE;
	    UpdateItem(15,NOMODIFY,SolderLayerCol);
	    break;
    }
}

void SelLabel(void)
{
    SetDrMd(RPd,JAM1);

    switch (LabelLayer)
    {
	case LAYER_ACTIVE:
	    LabelLayer = LAYER_HIDDEN;
	    UpdateItem(13,NOMODIFY,ClsCol);
	    break;

	case LAYER_HIDDEN:
	    LabelLayer = LAYER_ACTIVE;
	    UpdateItem(13,NOMODIFY,LabelLayerCol);
	    break;
    }
}

void SelActive(int Layer)
{
    SetDrMd(RPd,JAM1|COMPLEMENT);

    switch (ActiveLayer)
    {
	case COMPONENTLAYER:
	    UpdateItem(14,UNSELECTED,NOMODIFY);
	    break;

	case SOLDERLAYER:
	    UpdateItem(15,UNSELECTED,NOMODIFY);
	    break;

	case LABELLAYER:
	    UpdateItem(13,UNSELECTED,NOMODIFY);
	    break;
    }
    switch (Layer)
    {
	case COMPONENTLAYER:
	    UpdateItem(14,SELECTED,NOMODIFY);
	    if (ComponentLayer != LAYER_ACTIVE)
	    {
		ComponentLayer = LAYER_HIDDEN;
		SelComponent();
		DoRefresh();
	    }
	    break;

	case SOLDERLAYER:
	    UpdateItem(15,SELECTED,NOMODIFY);
	    if (SolderLayer != LAYER_ACTIVE)
	    {
		SolderLayer = LAYER_HIDDEN;
		SelSolder();
		DoRefresh();
	    }
	    break;

	case LABELLAYER:
	    UpdateItem(13,SELECTED,NOMODIFY);
	    if (LabelLayer != LAYER_ACTIVE)
	    {
		LabelLayer = LAYER_HIDDEN;
		SelLabel();
		DoRefresh();
	    }
	    break;
    }
    ActiveLayer = Layer;
}

void UpdateColors(void)
{
    UpdateItem(13,NOMODIFY,LabelLayerCol);
    UpdateItem(14,NOMODIFY,CompoLayerCol);
    UpdateItem(15,NOMODIFY,SolderLayerCol);
    UpdateItem(16,NOMODIFY,PadCol);
    UpdateItem(17,NOMODIFY,ConnectCol);
    UpdateItem(18,NOMODIFY,PartsCol);
}

void InitGadLink(struct Gadget *prev, struct Gadget *buff,x,y,width,height,id)
{
    short i;
    char  *ch;

    ch = (char *) buff;

    for (i=0; i<sizeof(struct Gadget); i++)
	ch[i]=0;

    prev->NextGadget = buff;
    buff->NextGadget = NULL;

    buff->LeftEdge = x;
    buff->TopEdge = y;
    buff->Width = width;
    buff->Height = height;
    buff->Flags = GADGHCOMP;
    buff->Activation = RELVERIFY;
    buff->GadgetType = BOOLGADGET;
    buff->GadgetRender = NULL;
    buff->SelectRender = NULL;
    buff->GadgetText = NULL;
    buff->MutualExclude = 0;
    buff->SpecialInfo = NULL;
    buff->GadgetID = id;
    buff->UserData = NULL;
}

short  DockSizeX, SelItem;

void OpenDock(void)
{
    short i, y;

    y = (myfont8.ta_YSize+3)*18;

    if (y > MaxScreenY)
	y = MaxScreenY;

    if (!(DockWin = OpenWindowTags(NULL,
		WA_Top,    myfont8.ta_YSize+3,
		WA_Left,   MaxScreenX - DockSizeX,
		WA_Width,  DockSizeX,
		WA_Height, y,
		WA_IDCMP,  IDCMP_INACTIVEWINDOW|IDCMP_MOUSEMOVE|IDCMP_RAWKEY|
			   IDCMP_VANILLAKEY|IDCMP_MOUSEBUTTONS,
		WA_Flags,  WFLG_REPORTMOUSE|WFLG_BORDERLESS|WFLG_RMBTRAP,
		WA_CustomScreen, MyScreen,
		TAG_END)))
	Ende(puts("ERROR: Kann Menu (Dock) Window nicht ffnen."));

    DockWin->BorderTop = 0;

    RPd = DockWin->RPort;

    dockx = DockSizeX-4;
    docky = MaxScreenY;
    fontheight = myfont8.ta_YSize;

    SetAPen(RPd,ClsCol);
    Move(RPd,0,0); Draw(RPd,DockSizeX,0);

    SolderLayer = LAYER_HIDDEN;
    SelSolder();
    ActiveLayer = SOLDERLAYER;

    ComponentLayer = LAYER_HIDDEN;
    SelComponent();

    LabelLayer = LAYER_HIDDEN;
    SelLabel();

    PadLayer = LAYER_HIDDEN;
    SelPadLayer();

    UpdateColors();

    i=1;
    while (DrawItem(i++));

    UpdateItem(2,SELECTED,NOMODIFY);
    SelItem = 2;
}

void CloseDock(void)
{
    if (DockWin)    CloseWindow(DockWin);
    DockWin = NULL;
}

struct Gadget *GetGadAddr(struct Window *GadWin, UWORD GadID)
{
    struct Gadget *gad = GadWin->FirstGadget;

    while (gad != NULL)
    {
	if (gad->GadgetID == GadID)
	    return (gad);
	gad = gad->NextGadget;

    }
    return (NULL);
}


void WrtPnt(void)
{
    if (Tool != PART_ADD)
    {
	SetDrMd(RP,JAM1|COMPLEMENT);
	WritePixel(RP,Zoll2X(kx),Zoll2Y(ky));
    }
}

void CompNewDPI(void)
{
    ActualdpiX = PCBZOOM * PCBBASEX / 100;
    ActualdpiY = PCBZOOM * PCBBASEY / 100;
}

void UpdateZoom(void)
{
    char txt[10];

    short z10 = PCBZOOM/100;
    short z1  = PCBZOOM-z10*100;

    sprintf(txt,"_Zoom %d.%01d",z10,z1);
    UpdateItemText(19,txt);
}

void ZoomPlus(void)
{
    if (PCBZOOM < 800)
    {
	HideIt();
	GetNextZF();
	CompNewDPI();
	UpdateZoom();
	PRKoords();
	DoRefresh();
	ShowIt();
    }
}

void ZoomMinus(void)
{
    if (PCBZOOM > 10)
    {
	HideIt();
	GetPrevZF();
	CompNewDPI();
	UpdateZoom();
	PRKoords();
	DoRefresh();
	ShowIt();
    }
}

void SelectGadSt(void)
{
    EmptyMsgPort();
}

void SelNewPartClear(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    CleanUp();
    Tool = PART_CLEAR;
    DoOnXY();
    UpdateItem(4,SELECTED,NOMODIFY);
    SelItem = 4;
    WrtPnt();
}

void SelNewPartCopy(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    CleanUp();
    Tool = PART_COPY;
    DoOnXY();
    PartEnd = PartStart;
    UpdateItem(5,SELECTED,NOMODIFY);
    SelItem = 5;
    WrtPnt();
}

void SelNewTrace(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    NewTool(IMG_LAYER,ADD_TRACE);
    UpdateItem(1,SELECTED,NOMODIFY);
    SelItem = 1;
    WrtPnt();
}

void SelNewPad(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    NewTool(IMG_LAYER,ADD_PAD);
    x2 = x;
    y2 = y;
    if (HideArea)
	HideFlag = TRUE;
    else
	DrawPad(x2,y2,PinSize,INVERS);

    UpdateItem(2,SELECTED,NOMODIFY);
    SelItem = 2;
    WrtPnt();
}

void SelNewUndo(void)
{
    struct Pad *TmpStart;
    ULONG TmpLen = (ULONG) PCBEnd - (ULONG) PCBStart+1;

    if ((TmpStart = (struct Pad *) AllocMem(PCBMEM_SIZE,MEMF_CLEAR)))
	CopyMem((char *) PCBStart,(char *) TmpStart, TmpLen);

    CopyMem((char *) UndoBuffer,(char *) PCBStart, (long) UndoEnd - (long) UndoBuffer + 1);
    (char *) PCBEnd = (char *) PCBStart + ((char *) UndoEnd - (char *) UndoBuffer);

    if (TmpStart)
    {
	CopyMem((char *) TmpStart,(char *) UndoBuffer, TmpLen);
	(long) UndoEnd = (long) UndoBuffer + TmpLen - 1;
	FreeMem(TmpStart, PCBMEM_SIZE);
    }
    DoRefresh();
    UpdateItem(9,UNSELECTED,NOMODIFY);
}

void SelNewLine(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    NewTool(IMG_LABEL,ADD_LINE);
    UpdateItem(10,SELECTED,NOMODIFY);
    SelItem = 10;
    WrtPnt();
}

void SelNewText(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    CleanUp();
    Tool = ADD_TEXT;
    InitDrawTxt();
    UpdateItem(6,SELECTED,NOMODIFY);
    SelItem = 6;
    WrtPnt();
}

void SelNewEllip(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    NewTool(IMG_LABEL,ADD_ELLIPSE);
    UpdateItem(12,SELECTED,NOMODIFY);
    SelItem = 12;
    WrtPnt();
}

void SelNewRect(void)
{
    UpdateItem(SelItem,UNSELECTED,NOMODIFY);
    NewTool(IMG_LABEL,ADD_RECT);
    UpdateItem(11,SELECTED,NOMODIFY);
    SelItem = 11;
    WrtPnt();
}

void SelNewTangle(void)
{
    char txt[10];

    HideIt();
    tangle += 45;
    if (tangle > 90)
	tangle = 0;
    ShowIt();

    sprintf(txt,"_W:Arc %d",tangle);
    UpdateItemText(22,txt);
}

void SelGridOnOff(void)
{
    if (PCBGRIDFLAG)
    {
	UpdateItem(20,UNSELECTED,NOMODIFY);
	PCBGRIDFLAG = FALSE;
    }
    else
    {
	PCBGRIDFLAG = TRUE;
	UpdateItem(20,SELECTED,NOMODIFY);
    }
    DoRefresh();
}

void DoNewStep(void)
{
    BOOL flag;

    GT_SetGadgetAttrs(GetGadAddr(Step_mmWnd,GD_CS_StepG),Step_mmWnd,NULL,
	GTCY_Active, PCBACTSTEP, TAG_END);

    InToStr(PCBSTEP, dummy);
    if (PCBACTSTEP == 9)
	flag = FALSE;
    else
	flag = TRUE;

    GT_SetGadgetAttrs(GetGadAddr(Step_mmWnd,GD_CS_CustomG),Step_mmWnd,NULL,
	GTST_String, dummy, GA_Disabled, flag, TAG_END);
}

void SwitchPCBStep(void)
{
    switch (PCBACTSTEP)
    {
	case 0:
	    PCBSTEP = StrToIn("5.08mm");
	    break;
	case 1:
	    PCBSTEP = StrToIn("2.54mm");
	    break;
	case 2:
	    PCBSTEP = StrToIn("1.27mm");
	    break;
	case 3:
	    PCBSTEP = StrToIn("0.635mm");
	    break;
	case 4:
	    PCBSTEP = StrToIn("5.0mm");
	    break;
	case 5:
	    PCBSTEP = StrToIn("2.0mm");
	    break;
	case 6:
	    PCBSTEP = StrToIn("1.0mm");
	    break;
	case 7:
	    PCBSTEP = StrToIn("0.8mm");
	    break;
	case 8:
	    PCBSTEP = StrToIn("0.65mm");
	    break;
	case 9:
	    break;
    }
}

void FastSelStep(void)
{
    if (++PCBACTSTEP > 8) PCBACTSTEP = 0;
    SwitchPCBStep();

    char txt1[20], txt2[20];

    strcpy(txt1,"Step ");
    InToStr(PCBSTEP,txt2);
    strcat(txt1,txt2);

    UpdateItemText(21,txt1);
}


void SelPadLayer(void)
{
    if (PadLayer == LAYER_HIDDEN)
    {
	PadLayer = LAYER_ACTIVE;
	UpdateItem(16,SELECTED,NOMODIFY);
    }
    else
    {
	PadLayer = LAYER_HIDDEN;
	UpdateItem(16,UNSELECTED,NOMODIFY);
    }
}

void SelNewStepSize(void)
{
    WORD gadnum;

    OpenStep_mmWindow();
    DoNewStep();

    do
    {
	Wait(1<<Step_mmWnd->UserPort->mp_SigBit);    // Auf Msg Warten

	Message = GT_GetIMsg(Step_mmWnd->UserPort);

	CopyMem((APTR) Message, (APTR) &MyMess, sizeof(struct IntuiMessage));
	GT_ReplyIMsg(Message);
	Message = &MyMess;

	gadnum = ((struct Gadget *) Message->IAddress)->GadgetID;

	switch (gadnum)
	{
	    case GD_CS_StepG:
		PCBACTSTEP = Message->Code;
		SwitchPCBStep();
		break;

	    case GD_CS_CustomG:
		PCBSTEP = StrToIn(((struct StringInfo *)((struct Gadget *) Message->IAddress)->SpecialInfo)->Buffer);
		break;
	}
	DoNewStep();
    }
    while ((gadnum != GD_CS_OkG) AND (gadnum != GD_CS_CancelG));

    CloseStep_mmWindow();

    char txt1[20], txt2[20];

    strcpy(txt1,"Step ");
    InToStr(PCBSTEP,txt2);
    strcat(txt1,txt2);

    UpdateItemText(21,txt1);
}

void DoNewGrid(void)
{
    BOOL flag;

    GT_SetGadgetAttrs(GetGadAddr(Grid_mmWnd,GD_CG_GridG),Grid_mmWnd,NULL,
	GTCY_Active, PCBACTGRID, TAG_END);

    InToStr(PCBGRID, dummy);
    if (PCBACTGRID == 9)
	flag = FALSE;
    else
	flag = TRUE;

    GT_SetGadgetAttrs(GetGadAddr(Grid_mmWnd,GD_CG_CustomG),Grid_mmWnd,NULL,
	GTST_String, dummy, GA_Disabled, flag, TAG_END);

    GT_SetGadgetAttrs(GetGadAddr(Grid_mmWnd,GD_CG_GridOnG),Grid_mmWnd,NULL,
	GTCB_Checked, PCBGRIDFLAG, TAG_END);
}

void SelNewGridRaster(void)
{
    WORD gadnum;

    OpenGrid_mmWindow();
    DoNewGrid();

    do
    {
	Wait(1<<Grid_mmWnd->UserPort->mp_SigBit);    // Auf Msg Warten

	Message = GT_GetIMsg(Grid_mmWnd->UserPort);

	CopyMem((APTR) Message, (APTR) &MyMess, sizeof(struct IntuiMessage));
	GT_ReplyIMsg(Message);
	Message = &MyMess;

	gadnum = ((struct Gadget *) Message->IAddress)->GadgetID;

	switch (gadnum)
	{
	    case GD_CG_GridG:
		PCBACTGRID = Message->Code;

		switch (PCBACTGRID)
		{
		    case 0:
			PCBGRID = StrToIn("5.08mm");
			break;
		    case 1:
			PCBGRID = StrToIn("2.54mm");
			break;
		    case 2:
			PCBGRID = StrToIn("1.27mm");
			break;
		    case 3:
			PCBGRID = StrToIn("0.635mm");
			break;
		    case 4:
			PCBGRID = StrToIn("5.0mm");
			break;
		    case 5:
			PCBGRID = StrToIn("2.0mm");
			break;
		    case 6:
			PCBGRID = StrToIn("1.0mm");
			break;
		    case 7:
			PCBGRID = StrToIn("0.8mm");
			break;
		    case 8:
			PCBGRID = StrToIn("0.65mm");
			break;
		    case 9:
			break;
		}
		break;

	    case GD_CG_CustomG:
		PCBGRID = StrToIn(((struct StringInfo *)((struct Gadget *) Message->IAddress)->SpecialInfo)->Buffer);
		break;

	    case GD_CG_GridOnG:
		PCBGRIDFLAG = !PCBGRIDFLAG;
		break;
	}
	DoNewGrid();
    }
    while ((gadnum != GD_CG_OkG) AND (gadnum != GD_CG_CancelG));

    CloseGrid_mmWindow();
    DoRefresh();

    char txt1[20], txt2[20];

    strcpy(txt1,"_Grid ");
    InToStr(PCBGRID,txt2);
    strcat(txt1,txt2);

    UpdateItemText(20,txt1);
}

void SelectDock(void)
{
    short id;

    id = GetIdXY(GetDockXPos(Message->MouseX),GetDockYPos(Message->MouseY));

    if (id == FALSE) return;

    if (Message->Code == IECODE_RBUTTON)
    {
	switch (id)
	{
	    case 13:
		SelLabel();
		DoRefresh();
		break;
	    case 14:
		SelComponent();
		DoRefresh();
		break;
	    case 15:
		SelSolder();
		DoRefresh();
		break;
	    case 19:
		ZoomMinus();
		break;
	    case 20:
		SelNewGridRaster();
		break;
	    case 21:
		SelNewStepSize();
		break;
	}
    }

    if (Message->Code == SELECTDOWN)
    {
	switch (id)
	{
	    case 1:
		SelNewTrace();
		break;
	    case 2:
		SelNewPad();
		break;
	    case 4:
		if (SelectFlag == 1)
		    Delete();
		else
		    SelNewPartClear();
		break;
	    case 5:
		SelNewPartCopy();
		break;
	    case 6:
		SelNewText();
		break;
	    case 9:
		SelNewUndo();
		break;
	    case 10:
		SelNewLine();
		break;
	    case 11:
		SelNewRect();
		break;
	    case 12:
		SelNewEllip();
		break;
	    case 13:
		SelActive(LABELLAYER);
		break;
	    case 14:
		SelActive(COMPONENTLAYER);
		break;
	    case 15:
		SelActive(SOLDERLAYER);
		break;
	    case 16:
		SelPadLayer();
		DoRefresh();
		break;
	    case 19:
		ZoomPlus();
		break;
	    case 20:
		SelGridOnOff();
		break;
	    case 21:
		FastSelStep();
		break;
	    case 22:
		SelNewTangle();
		break;
	}
    }
    EmptyMsgPort();
}

void DoSorry(void)
{
    rtEZRequest("I am really sorry,\n but this feature is not available yet.\n\nPlease watch out for new versions of CrazyPads !","  OK  ",NULL,(struct TagItem *) mtags);
}

void MoveLeft(void)
{
    short scrxstep = ((short) ((long) ScrLimXend * 160000 / ActualdpiX / 300));
    // short dxx = Zoll2PixX(scrxstep);

    if (PCBOFFSETX-scrxstep > -32767)
    {
	HideIt();
	PCBOFFSETX -= scrxstep;
	/*
	    SetBPen(RP,1);
	    ScrollRaster(RP,-dxx,0,0,StatusHeight, ScrLimXend,ScrLimYend);
	    RefreshRegion(0,StatusHeight,dxx,ScrLimYend,FALSE);
	*/
	DoRefresh();
	ShowIt();
    }
}

void MoveRight(void)
{
    short scrxstep = ((short) ((long) ScrLimXend * 160000 / ActualdpiX / 300));
    // short dxx = Zoll2PixX(scrxstep);

    if (PCBOFFSETX < 32768-scrxstep)
    {
	HideIt();
	PCBOFFSETX += scrxstep;
	/*
	    SetBPen(RP,1);
	    ScrollRaster(RP,dxx,0,0,StatusHeight, ScrLimXend,ScrLimYend);
	    RefreshRegion(ScrLimXend-dxx,StatusHeight,ScrLimXend,ScrLimYend,FALSE);
	*/
	DoRefresh();
	ShowIt();
    }
}

void MoveUp(void)
{
    short scrystep = ((short) ((long) ScrLimYend * 160000 / ActualdpiY / 300));
    // short dyy = Zoll2PixY(scrystep);

    if (PCBOFFSETY-scrystep > -32767)
    {
	HideIt();
	PCBOFFSETY -= scrystep;
	/*
	    SetBPen(RP,1);
	    ScrollRaster(RP,0,-dyy,0,StatusHeight, ScrLimXend,ScrLimYend);
	    RefreshRegion(0,StatusHeight,ScrLimXend,dyy,FALSE);
	*/
	DoRefresh();
	ShowIt();
    }
}

void MoveDown(void)
{
    short scrystep = ((short) ((long) ScrLimYend * 160000 / ActualdpiY / 300));
    // short dyy = Zoll2PixY(scrystep);

    if (PCBOFFSETY < 32768-scrystep)
    {
	HideIt();
	PCBOFFSETY += scrystep;
	/*
	    SetBPen(RP,1);
	    ScrollRaster(RP,0,dyy,0,StatusHeight, ScrLimXend,ScrLimYend);
	    RefreshRegion(0,ScrLimYend-dyy,ScrLimXend,ScrLimYend,FALSE);
	*/
	DoRefresh();
	ShowIt();
    }
}

void SelectRawKey(void)
{
    switch (Message->Code)
    {
	case 0x4f:		/* Crs Left */
	    MoveLeft();
	    break;

	case 0x4e:		/* Crs Right */
	    MoveRight();
	    break;

	case 0x4c:		/* Crs Up */
	    MoveUp();
	    break;

	case 0x4d:		/* Crs Down */
	    MoveDown();
	    break;

	case 0x5F:	       /* HELP */
	    Execute("run >nil: <nil: AmigaGuide s:CrazyPads.guide PubScreen CrazyPads-PubScr",0L,0L);
	    break;
    }
}

void SelectVanillaKey(void)
{
    int sw;

    sw = Message->Code;

    switch (sw)
    {
	case '+':              /* + */
	    ZoomPlus();
	    break;

	case '-':              /* - */
	    ZoomMinus();
	    break;

	case 'b':              /* b = Label Layer */
	    SelLabel();
	    DoRefresh();
	    break;

	case 'B':              /* B = Label Layer */
	    SelActive(LABELLAYER);
	    DoRefresh();
	    break;

	case 'd':
	    if (SelectFlag == 1)
		Delete();
	    else
		SelNewPartClear();
	    break;

	case 'f':
	    FastSelStep();
	    break;

	case 'F':
	    SelNewStepSize();
	    break;

	case 'm':              /* m = Component Layer */
	    SelComponent();
	    DoRefresh();
	    break;

	case 'M':
	    SelActive(COMPONENTLAYER);
	    break;

	case 'i':              /* i = Ellipsen Zeichnen */
	    SelNewEllip();
	    break;

	case 'g':              /* G */
	    SelGridOnOff();
	    break;

	case 'G':              /* G */
	    SelNewGridRaster();
	    break;

	case 'k':              /* K */
	    if (PCBXYFLAG == TRUE)
		DoOffXY();
	    else
		DoOnXY();
	    break;

	case 'l':
	    SelPadLayer();
	    break;

	case 'n':              /* n = Linien Zeichnen */
	    SelNewLine();
	    break;

	case 'p':              /* p = Pads zeichnen */
	    SelNewPad();
	    break;

	case 'r':              /* r = Rechtecke zeichnen */
	    SelNewRect();
	    break;

	case ' ':              /* SPACE */
	    DoRefresh();
	    break;

	case 's':              /* s = Solder Layer */
	    SelSolder();
	    DoRefresh();
	    break;

	case 'S':              /* S = Solder Layer */
	    SelActive(SOLDERLAYER);
	    break;

	case 't':               // T = Leiterbahnen (Traces) zeichnen
	    SelNewTrace();
	    break;

	case 'u':               // U = Undo
	    SelNewUndo();
	    break;

	case 'w':
	    SelNewTangle();
	    break;

	case 'x':               // X: Text einzeichnen
	    SelNewText();
	    break;

	case 'y':
	    SelNewPartCopy();
	    break;

	case '0':
	    ChangeSize(10);
	    break;
    }
    if (sw > 48 AND sw < 58)	     // Zahlen '1' bis '9'
	ChangeSize((short) sw-48);   // Codes 1 bis 9:
				     // Code '0' im Menu
    PRKoords();
}
