/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     label.c
 * CONTENTS: Routines for drawing label layer
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "/obj/prog-protos.h"

Prototype void DoLine(short x_start, short y_start, short x_end, short y_end, short thickness, short mode);
Prototype void DoEllipse(short x_pos, short y_pos, short size_x, short size_y, short thickness, short mode);
Prototype void DoRect(short x_pos, short y_pos, short x1_pos, short y1_pos, short thickness, short mode);
Prototype struct Line *AddLine(short x_pos,short y_pos,short x1_pos,short y1_pos,short thickness);
Prototype struct Ellipse *AddEllipse(short x_pos,short y_pos,short size_x,short size_y,short thickness);
Prototype struct Rect *AddRect(short x_pos,short y_pos,short x1_pos,short y1_pos,short thickness);
Prototype void SetLine(void);
Prototype void SetEllipse(void);
Prototype void SetRect(void);
Prototype void SetCircle(void);

Prototype void OffXY(void);
Prototype void OnXY(void);

void DoLine(short x_start, short y_start, short x_end, short y_end, short thickness, short mode)
{
    x_start = Zoll2X(x_start);
    y_start = Zoll2Y(y_start);
    x_end = Zoll2X(x_end);
    y_end = Zoll2Y(y_end);

    if ((x_start<ScrLimXstart || x_start>ScrLimXend || y_start<ScrLimYstart || y_start>ScrLimYend) && (x_end<ScrLimXstart || x_end>ScrLimXend || y_end<ScrLimYstart || y_end>ScrLimYend))
	return;

    if (mode == INVERS || mode == SELECT)
	SetDrMd(RP,JAM1|COMPLEMENT);
    else if (mode == CLEAR) {
	SetDrMd(RP,JAM1);
	SetAPen(RP,ClsCol);
    }
    else SetLayerColor(LABELLAYER);

    DrawLine(x_start,y_start,x_end,y_end);
/*
    if (thickness>0)
	DrawLine(x_start+1,y_start,x_end+1,y_end);
*/
    return;
}

void DoEllipse(short x_pos, short y_pos, short size_x, short size_y, short thickness, short mode)
{
    x_pos = Zoll2X(x_pos);
    y_pos = Zoll2Y(y_pos);

    size_x = Zoll2PixX(size_x);
    size_y = Zoll2PixY(size_y);

    short x_start = x_pos+size_x;
    short y_start = y_pos+size_y;

    short x_end = x_pos-size_x;
    short y_end = y_pos-size_y;

    if ((x_start<ScrLimXstart || x_start>ScrLimXend || y_start<ScrLimYstart || y_start>ScrLimYend) && (x_end<ScrLimXstart || x_end>ScrLimXend || y_end<ScrLimYstart || y_end>ScrLimYend))
	return;

    if (mode == INVERS || mode == SELECT)
	SetDrMd(RP,JAM1|COMPLEMENT);
    else if (mode == CLEAR) {
	SetDrMd(RP,JAM1);
	SetAPen(RP,ClsCol);
    }
    else SetLayerColor(LABELLAYER);

    DrawEllipse(RP,x_pos,y_pos,size_x,size_y);
/*
    if (thickness>0)
	DrawEllipse(RP,x_pos+1,y_pos,size_x,size_y);
*/
    return;
}

void DoRect(short x_pos, short y_pos, short x1_pos, short y1_pos, short thickness, short mode)
{
    x_pos = Zoll2X(x_pos);
    y_pos = Zoll2Y(y_pos);
    x1_pos = Zoll2X(x1_pos);
    y1_pos = Zoll2Y(y1_pos);

    if ((x_pos<ScrLimXstart || x_pos>ScrLimXend || y_pos<ScrLimYstart || y_pos>ScrLimYend) && (x1_pos<ScrLimXstart || x1_pos>ScrLimXend || y1_pos<ScrLimYstart || y1_pos>ScrLimYend))
	return;

    if (mode == INVERS || mode == SELECT)
	SetDrMd(RP,JAM1|COMPLEMENT);
    else if (mode == CLEAR) {
	SetDrMd(RP,JAM1);
	SetAPen(RP,ClsCol);
    }
    else SetLayerColor(LABELLAYER);

    Move(RP,x_pos,y_pos);
    Draw(RP,x1_pos,y_pos); Draw(RP,x1_pos,y1_pos);
    Draw(RP,x_pos,y1_pos); Draw(RP,x_pos,y_pos);
/*
    Move(RP,x_pos+1,y_pos); Draw(RP,x_pos+1,y1_pos);
    Move(RP,x1_pos+1,y_pos); Draw(RP,x1_pos+1,y1_pos);
*/
    if (mode == INVERS)
	WritePixel(RP,x_pos,y_pos);

}

struct Line *AddLine(short x_pos,short y_pos,short x1_pos,short y1_pos,\
		     short thickness)
{
    struct Line *MyLine;

    MyLine = (struct Line *) PCBEnd;
    if ((char *) MyLine + sizeof(struct Line) > (char *) PCBMax)
	return(0);
    UpdateUndo();
    MyLine->Type = LABEL_ID;
    MyLine->Item = LINE_ID;
    MyLine->Thickness = thickness;
    MyLine->x_start = x_pos;
    MyLine->y_start = y_pos;
    MyLine->x_end =  x1_pos;
    MyLine->y_end =  y1_pos;
    (char *) PCBEnd += sizeof (struct Line);
    PCBEnd->Type = 0;
    ModCnt++;
    return(MyLine);
}

struct Ellipse *AddEllipse(short x_pos,short y_pos,short size_x,\
			   short size_y,short thickness)
{
    struct Ellipse *MyEllipse;

    MyEllipse = (struct Ellipse *) PCBEnd;
    if ((char *) MyEllipse + sizeof(struct Ellipse) > (char *) PCBMax)
	return(0);
    UpdateUndo();
    MyEllipse->Type = LABEL_ID;
    MyEllipse->Item = ELLIPSE_ID;
    MyEllipse->Thickness = thickness;
    MyEllipse->x_pos = x_pos;
    MyEllipse->y_pos = y_pos;
    MyEllipse->size_x =  size_x;
    MyEllipse->size_y =  size_y;
    (char *) PCBEnd += sizeof (struct Ellipse);
    PCBEnd->Type = 0;
    ModCnt++;
    return(MyEllipse);
}

struct Rect *AddRect(short x_pos,short y_pos,short x1_pos,short y1_pos,\
		     short thickness)
{
    struct Rect *MyRect;

    MyRect = (struct Rect *) PCBEnd;
    if ((char *) MyRect + sizeof(struct Rect) > (char *) PCBMax)
	return(0);
    UpdateUndo();
    MyRect->Type = LABEL_ID;
    MyRect->Item = RECT_ID;
    MyRect->Thickness = thickness;
    MyRect->x_pos = x_pos;
    MyRect->y_pos = y_pos;
    MyRect->x1_pos = x1_pos;
    MyRect->y1_pos = y1_pos;
    (char *) PCBEnd += sizeof (struct Rect);
    PCBEnd->Type = 0;
    ModCnt++;
    return(MyRect);
}

Local short RememberFlag;

void OffXY(void)
{
    if (PCBXYFLAG) {
	DrawXY(kx,ky);
	RememberFlag = 1;
	PCBXYFLAG = 0;
    }
}

void OnXY(void)
{
    if (RememberFlag) {
	kx = x;
	ky = y;
	PCBXYFLAG = 1;
	RememberFlag = 0;
	DrawXY(kx,ky);
    }
}

void SetLine(void)
{
    if (Message->Class == MOUSEBUTTONS) {
	if (Message->Code==SELECTDOWN && !DrawKey) {
	    x1=x2=x;
	    y1=y2=y;
	    DrawKey = 1;
	    OffXY();
	}
	else if (Message->Code==SELECTDOWN && DrawKey==1) {
	    OnXY();
	    if (!(x1==x2 && y1==y2)) {
		DoLine(x1,y1,x2,y2,1,LabelLayerCol);
		AddLine(x1,y1,x2,y2,1);
	    }
	    DrawKey = 0;
	}
    }
    else if (Message->Class==MOUSEMOVE && DrawKey==1) {
	DoLine(x1,y1,x2,y2,0,INVERS);
	x2 = x;
	y2 = y;
	DoLine(x1,y1,x2,y2,0,INVERS);
    }
}

void SetEllipse(void)
{
    if (Message->Class==MOUSEMOVE && DrawKey == 1) {
	DoEllipse(x1,y1,x2,y2,0,INVERS);    /* alten Kreis lschen */
	x2 = ABS(x-x1);
	y2 = ABS(y-y1);
	DoEllipse(x1,y1,x2,y2,0,INVERS);
    }
    else if (Message->Class==MOUSEBUTTONS) {
	if (Message->Code==SELECTDOWN && DrawKey == 0) {
	    x1 = x;
	    y1 = y;
	    y2 = x2 = 0;
	    DrawKey = 1;
	    DoEllipse(x1,y1,x2,y2,0,INVERS);	/* Kreis zeichnen */
	    OffXY();
	}
	else if (Message->Code == SELECTDOWN && DrawKey == 1) {
	    if (!(x2==0 && y2==0)) {
		DoEllipse(x1,y1,x2,y2,1,LabelLayerCol);
		AddEllipse(x1,y1,x2,y2,1);
	    }
	    else DoEllipse(x1,y1,x2,y2,0,INVERS);
	    DrawKey = 0;
	    OnXY();
	}
    }
}

void SetRect(void)
{
    if (Message->Class == MOUSEBUTTONS) {
	if (Message->Code==SELECTDOWN && DrawKey==0) {
	    x1=x2=x;
	    y1=y2=y;
	    DrawKey = 1;
	    OffXY();
	}
	else if (Message->Code==SELECTDOWN && DrawKey==1) {
	    if (!(x1==x2 && y1==y2)) {
		DoRect(x1,y1,x2,y2,1,LabelLayerCol);
		AddRect(x1,y1,x2,y2,1);
	    }
	    DrawKey = 0;
	    OnXY();
	}
    }
    if (Message->Class==MOUSEMOVE && DrawKey) {
	DoRect(x1,y1,x2,y2,0,INVERS);
	x2 = x;
	y2 = y;
	DoRect(x1,y1,x2,y2,0,INVERS);
    }
}

void SetCircle(void)
{}
