/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     layer.c
 * CONTENTS:
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "/obj/prog-protos.h"

Prototype void GetNextZF(void);
Prototype void GetPrevZF(void);
Prototype long GetEntrySize(struct Element *Element);
Prototype void *GetNextEntry(struct Element *Element);
Prototype void *FindEntry(short x_pos, short y_pos, short type);
Prototype void *GetNextElement(short Element);
Prototype void *GetFirstElement(short Element);
Prototype int DelEntry(struct Element *MyElement);
Prototype int CopyEntry(struct Element *Src, APTR *Dest);
Prototype struct Pad *AddPad(short pinsize, short x_pos, short y_pos);
Prototype struct Pad *FindPad(short x_pos, short y_pos);
Prototype struct Pad *DelPad(short x_pos, short y_pos);
Prototype struct Trace *FindTrPnt(short x_pos, short y_pos);
Prototype struct Trace *FindTraceLayer(short x_pos,short y_pos,short x1_pos, short y1_pos,short layer);
Prototype struct Trace *AddTrace(short size,short x_pos,short y_pos, short x1_pos,short y1_pos,short layer);
Prototype struct Trace *DelTrace(short x_pos,short y_pos,short x1_pos, short y1_pos,short layer);
Prototype void RefreshScreen(void);
Prototype void DrawEntry(struct Element *MyElement, short mode);
Prototype void DoSelect(void);
Prototype int Delete(void);
Prototype void MoveIt(void);
Prototype void Draw45Trace(short mode);
Prototype void Draw45(short mode);
Prototype void SetTrace(void);
Prototype void SetPad(void);
Prototype short StrToIn(char *str);
Prototype void InToStr(short in, char *str);

Prototype short SelectFlag;
Prototype short tangle;

Prototype struct Element *SelectElement;

Prototype short MoveFlag,move_x,move_y;


Local void SelElement(struct Element *element);

struct Element *SelectElement;
short SelectFlag;
short SelectX, SelectY;

void GetNextZF(void)
{
    short i;
    i=0;

    do
    {
	if (PCBZOOM == Zfak[i] AND Zfak[i+1] != 0)
	    PCBZOOM = Zfak[++i];
    }
    while (Zfak[++i] != 0);
}

void GetPrevZF(void)
{
    short i;
    i=1;

    do
    {
	if (PCBZOOM == Zfak[i])
	    PCBZOOM = Zfak[i-1];
    }
    while (Zfak[++i] != 0);
}

short StrToIn(char *str)
{
    double buf;
    char txt[40];
    short ts;
    long tl;

    sscanf(str,"%lf%2s",&buf,txt);

    //					    25.4 [mm]	 (oder 2.54cm, oder 0.0254m)
    // result [1/1600in] := eingabe [mm] / -----------
    //					   1600 [1/in]

    //					     1 [in]    (oder 2.54cm, oder 0.0254m)
    // result [1/1600in] := eingabe [in] / -----------
    //					   1600 [1/in]

    if (!strcmp(txt,"mm"))      // short (1/1600in) := buf / 0.015875;
	tl = IEEESPFix( buf / 0.015875 + 0.05);
    else if (!strcmp(txt,"cm"))
	tl = IEEESPFix( buf / 0.0015875);
    else if (!strcmp(txt,"m"))
	tl = IEEESPFix( buf / 0.000015875);
    else if (!strcmp(txt,"in"))
	tl = IEEESPFix( buf / 0.000625);
    else
	ts = -1;

    if (tl < -1)
	ts = 0;
    else if (tl > 32768)
	ts = 32768;
    else
	ts = tl;

    return(ts);
}

void InToStr(short in, char *str)
{
    //					     1 [in]    (oder 2.54cm, oder 0.0254m)
    // result [1/1600in] := eingabe [in] / -----------
    //					   1600 [1/in]

    sprintf(str,"%3.02gmm",(float) in * 0.015875);
    return;
}

long GetEntrySize(struct Element *Element)
{
    struct Text *mytext;
    struct Part *mypart;
    struct TraceEntry *mytraces;

    if (Element->Type != 0)
    {
	switch (Element->Type)
	{
	    case PAD_ID:
		return(sizeof(struct Pad));

	    case TRACE_ID:
		return(sizeof(struct Trace));

	    case TEXT_ID:
		mytext = (struct Text *) Element;
		return(sizeof(struct Text)+mytext->Textsize);

	    case PART_ID:
		mypart = (struct Part *) Element;
		return(sizeof(struct Part)+mypart->Size);

	    case TRACEENTRY_ID:
		mytraces = (struct TraceEntry *) Element;
		return(sizeof(struct TraceEntry)+mytraces->Size);

	    case LABEL_ID:
		switch (Element->Item)
		{
		    case LINE_ID:
			return(sizeof(struct Line));

		    case ELLIPSE_ID:
			return(sizeof(struct Ellipse));
/*
		    case CIRCLE_ID:
			return(sizeof(struct Circle));
*/
		    case RECT_ID:
			return(sizeof(struct Rect));
			break;
		}

	    default:
		return(0);
	}
    }
    return(0);
}

void *FindEntry(short x_pos, short y_pos, short type)
{
    union elements my;

    my.element = (struct Element *) PCBStart;

    do
    {
	switch (my.element->Type)
	{
	    case PAD_ID:
		if (type != PAD_ID AND type != 0) break;

		if (my.pad->x_pos==x_pos AND my.pad->y_pos == y_pos)
		    return(my.element);
		break;

	    case TRACE_ID:
		if (type != TRACE_ID AND type != 0) break;

		if ((my.trace->x_start==x_pos AND my.trace->y_start==y_pos) \
		    OR (my.trace->x_end==x_pos AND my.trace->y_end==y_pos))
		    return(my.element);
		break;

	    case TEXT_ID:
		if (type != TEXT_ID AND type != 0) break;

		if (my.text->x_pos==x_pos AND my.text->y_pos==y_pos)
		    return(my.element);
		break;

	    case LABEL_ID:
		if (type != LABEL_ID AND type != 0) break;

		switch (my.element->Item)
		{
		    case LINE_ID:
			if ((my.line->x_start==x_pos AND my.line->y_start==y_pos) \
			    OR (my.line->x_end==x_pos AND my.line->y_end==y_pos))
			    return(my.element);
			break;

		    case RECT_ID:
			if ((my.rect->x_pos==x_pos AND my.rect->y_pos==y_pos) \
			    OR (my.rect->x1_pos==x_pos AND my.rect->y1_pos==y_pos))
			    return(my.element);
			break;

		    case ELLIPSE_ID:
			if (my.ellipse->x_pos==x_pos AND my.ellipse->y_pos==y_pos)
			    return(my.element);
			break;
		}
		break;
	}
    } while ((my.element = GetNextEntry(my.element)) != 0);
}

void *GetNextEntry(struct Element *Element)
{
    struct Element *elem;

    if (Element->Type == 0)
	return(NULL);

    (char *) elem = (char *) Element + GetEntrySize(Element);

    if (Element->Type == 0)
	return(NULL);

    return(elem);
}

struct Element *NextElement;

void *GetNextElement(short Element)
{
    short	   Flag;
    struct Element *Dummy;

    Flag = FALSE;
    Dummy = NULL;

    while (NextElement->Type != 0)
    {
	if (NextElement->Type == Element)
	{
	       Dummy = NextElement;
	       Flag = TRUE;
	}
	NextElement = (struct Element *) GetNextEntry(NextElement);
	if (Flag == TRUE)
	    return((void *) Dummy);
    }
    NextElement = (struct Element *) PCBStart;
    return(0);
}

void *GetFirstElement(short Element)
{
    NextElement = (struct Element *) PCBStart;
	/* diesmal von Anfang an suchen */
    return(GetNextElement(Element));
}

int DelEntry(struct Element *MyElement)
{
    long EntrySize = GetEntrySize(MyElement);

    if (EntrySize)
    {
	UpdateUndo();
	ModCnt++;
	CopyMem((char *) MyElement + EntrySize, (char *) MyElement, (long) PCBEnd - (long) MyElement);
	(char *) PCBEnd -= EntrySize;
	return(EntrySize);
    }
    return(0);
}

int CopyEntry(struct Element *Src, APTR *Dest)
{
    long EntrySize = GetEntrySize(Src);

    if (EntrySize)
    {
	CopyMem((char *) Src, Dest, EntrySize);
	return(EntrySize);
    }
    return(0);
}

struct Pad *AddPad(short pinsize, short x_pos, short y_pos)
{
    struct Pad *MyPad;

    MyPad = PCBEnd;
    if ((char *) MyPad + sizeof(struct Pad) > (char *) PCBMax)
	return(0);
    UpdateUndo();
    MyPad->Type = PAD_ID;
    MyPad->Size = pinsize;
    MyPad->x_pos = x_pos;
    MyPad->y_pos = y_pos;
    (char *) PCBEnd += sizeof(struct Pad);
    PCBEnd->Type = 0;
    ModCnt++;
    return(PCBEnd);
}

struct Pad *FindPad(short x_pos, short y_pos)
{
    struct Pad *MyPad;

    if (MyPad = (struct Pad *) GetFirstElement(PAD_ID)) /* Zeiger aufs erste PAD */
    {
	do
	{
	    if (MyPad->x_pos == x_pos AND MyPad->y_pos == y_pos) /* hat PAD die gewuenschten Koordinaten */
		return(MyPad); /* falls ja, return */
	}
	while ((MyPad = (struct Pad *) GetNextElement(PAD_ID)) != 0); /* alle PAD's durchlaufen */
    }
    return(0);
}

struct Pad *DelPad(short x_pos, short y_pos)
{
    struct Pad *MyPad;

    if (!(MyPad = FindPad(x_pos,y_pos)))
	return(0);
    else
    {
	UpdateUndo();
	CopyMem((char *) MyPad + sizeof(struct Pad),(char *) MyPad,\
	    (long) PCBEnd - (long) MyPad);
	(char *) PCBEnd -= sizeof(struct Pad);
	ModCnt++;
    }
    return(MyPad);
}

struct Trace *FindTrPnt(short x_pos, short y_pos)
{
    struct Trace *MyTrace;

    if (MyTrace = (struct Trace *) GetFirstElement(TRACE_ID)) /* Zeiger auf die erste TRACE */
    {
	do
	{
	    if (MyTrace->x_start == x_pos AND MyTrace->y_start == y_pos)
		return(MyTrace);
	    else if (MyTrace->x_end == x_pos AND MyTrace->y_end == y_pos)
		return(MyTrace);
	}
	while ((MyTrace = (struct Trace *) GetNextElement(TRACE_ID)) != 0);
    } /* alle TRACE's durchlaufen */
    return(0);
}

struct Trace *FindTrPntLayer(short x_pos, short y_pos, short layer)
{
    struct Trace *MyTrace;

    if (MyTrace = (struct Trace *) GetFirstElement(TRACE_ID)) // Zeiger auf die erste TRACE
    {
	do
	{
	    if (MyTrace->Layer == layer)
	    {
		if (MyTrace->x_start == x_pos AND MyTrace->y_start == y_pos)
		    return(MyTrace);
		else if (MyTrace->x_end == x_pos AND MyTrace->y_end == y_pos)
		    return(MyTrace);
	    }
	}
	while ((MyTrace = (struct Trace *) GetNextElement(TRACE_ID)) != 0);
    }
    return(0);
}

struct Trace *FindTraceLayer(short x_pos,short y_pos,short x1_pos, short y1_pos,short layer)
{
    struct Trace *MyTrace;

    if (MyTrace = (struct Trace *) GetFirstElement(TRACE_ID))
    {
	    /* Zeiger auf die erste TRACE */
	do
	{
	    if (MyTrace->x_start == x_pos AND MyTrace->y_start == y_pos\
		    AND MyTrace->x_end == x1_pos AND MyTrace->y_end == y1_pos\
		    AND MyTrace->Layer == layer)
		return(MyTrace);
	    else if (MyTrace->x_start == x1_pos AND MyTrace->y_start == y1_pos\
		    AND MyTrace->x_end == x_pos AND MyTrace->y_end == y_pos\
		    AND MyTrace->Layer == layer)
		return(MyTrace);
	}
	while ((MyTrace = (struct Trace *) GetNextElement(TRACE_ID)) != 0);
	    /* alle TRACE's durchlaufen */
    }
    return(0);
}

struct Trace *AddTrace(short size,short x_pos,short y_pos,\
		       short x1_pos,short y1_pos,short layer)
{
    struct Trace *MyTrace;

    MyTrace = (struct Trace *) PCBEnd;
    if ((char *) MyTrace + sizeof(struct Trace) > (char *) PCBMax)
	return(0);
    UpdateUndo();
    MyTrace->Type = TRACE_ID;
    MyTrace->Layer = layer;
    MyTrace->Size = size;
    MyTrace->x_start = x_pos;
    MyTrace->y_start = y_pos;
    MyTrace->x_end = x1_pos;
    MyTrace->y_end = y1_pos;
    (char *) PCBEnd += sizeof (struct Trace);
    PCBEnd->Type = 0;
    ModCnt++;
    return(MyTrace);
}

struct Trace *DelTrace(short x_pos,short y_pos,short x1_pos,\
		       short y1_pos,short layer)
{
    struct Trace *MyTrace;

    if (!(MyTrace = FindTraceLayer(x_pos,y_pos,x1_pos,y1_pos,layer)))
	return(0);
    else
    {
	UpdateUndo();
	CopyMem((char *) MyTrace + sizeof(struct Trace),(char *) MyTrace,\
	    (long) PCBEnd - (long) MyTrace);
	(char *) PCBEnd -= sizeof(struct Trace);
	ModCnt++;
    }
    return(MyTrace);
}

void RefreshScreen(void)
{
    struct Element  *MyElement;
    struct Pad	    *MyPad;
    struct Trace    *MyTrace;
    struct Text     *MyText;
    struct Line     *MyLine;
    struct Ellipse  *MyEllipse;
    struct Rect     *MyRect;

    char *text;

/* Zeichnet den inaktiven Layer */
    if (MyTrace = (struct Trace *) GetFirstElement(TRACE_ID))
    {
	do
	{
	    if (MyTrace->Layer != ActiveLayer)
		DrawTrace(MyTrace->x_start, MyTrace->y_start, MyTrace->x_end,\
		    MyTrace->y_end, MyTrace->Size, MyTrace->Layer);
	}
	while ((MyTrace = (struct Trace *) GetNextElement(TRACE_ID)) != 0);
    }

/* Zeichnet den aktiven Layer */

    if (MyTrace = (struct Trace *) GetFirstElement(TRACE_ID))
    {
	do
	{
	    if (MyTrace->Layer == ActiveLayer)
		DrawTrace(MyTrace->x_start, MyTrace->y_start, MyTrace->x_end,\
		    MyTrace->y_end, MyTrace->Size, MyTrace->Layer);
	}
	while ((MyTrace = (struct Trace *) GetNextElement(TRACE_ID)) != 0);
    }

/* Zeichnet den Label Layer (Bestueckungsaufdruck */

    if (MyElement = (struct Element *) GetFirstElement(LABEL_ID))
    {
	if (LabelLayer == LAYER_ACTIVE)
	{
	    do
	    {
		SetAPen(RP,2);
		switch (MyElement->Item)
		{
		    case LINE_ID:
			MyLine = (struct Line *) MyElement;
			DoLine(MyLine->x_start,MyLine->y_start,MyLine->x_end,\
			    MyLine->y_end,MyLine->Thickness,LabelLayerCol);
			break;
		    case ELLIPSE_ID:
			MyEllipse = (struct Ellipse *) MyElement;
			DoEllipse(MyEllipse->x_pos,MyEllipse->y_pos,\
			    MyEllipse->size_x,MyEllipse->size_y,\
			    MyEllipse->Thickness,LabelLayerCol);
			break;
		    case RECT_ID:
			MyRect = (struct Rect *) MyElement;
			DoRect(MyRect->x_pos,MyRect->y_pos,MyRect->x1_pos,\
			    MyRect->y1_pos,MyRect->Thickness,LabelLayerCol);
			break;
		}
	    }
	    while ((MyElement = (struct Element *) GetNextElement(LABEL_ID)) != 0);
	}
    }
    if (MyText = (struct Text *) GetFirstElement(TEXT_ID))
    {
	do
	{
	    text = (char *) MyText + sizeof(struct Text);
	    switch (MyText->Layer)
	    {
		case SOLDERLAYER:
		    if (SolderLayer != LAYER_HIDDEN)
			DrawVecTxt(RP,MyText->x_pos,MyText->y_pos,text,MyText->Size,MyText->Angle,MyText->Layer);
		    break;
		case COMPONENTLAYER:
		    if (ComponentLayer != LAYER_HIDDEN)
			DrawVecTxt(RP,MyText->x_pos,MyText->y_pos,text,MyText->Size,MyText->Angle,MyText->Layer);
		    break;
		case LABELLAYER:
		    if (LabelLayer != LAYER_HIDDEN)
			DrawVecTxt(RP,MyText->x_pos,MyText->y_pos,text,MyText->Size,MyText->Angle,MyText->Layer);
		    break;
	    }
	}
	while ((MyText = (struct Text *) GetNextElement(TEXT_ID)) != 0);
    }

/* Zeichnet die Pads */

    if (PadLayer == LAYER_ACTIVE)
    {
	if (MyPad = (struct Pad *) GetFirstElement(PAD_ID))
	{
	    do
	    {
		DrawPad(MyPad->x_pos, MyPad->y_pos, MyPad->Size, (short) PadCol);
	    }
	    while ((MyPad = (struct Pad *) GetNextElement(PAD_ID)) != 0);
	}
    }
}

void DrawEntry(struct Element *MyElement, short mode)
{
    union elements my;
    char  *txt;
    short size;

    my.element = MyElement;

    switch (my.element->Type)
    {
	case PAD_ID:
	    if (mode == NORM)
		mode = PadCol;

	    DrawPad(my.pad->x_pos,my.pad->y_pos,my.pad->Size, mode);
	    break;

	case TRACE_ID:
	    if (mode == NORM)
		mode = my.trace->Layer+CP_COMPLETE;

	    DrawTrace(my.trace->x_start, my.trace->y_start, my.trace->x_end,\
		      my.trace->y_end, my.trace->Size, mode);
	    break;
	case TEXT_ID:
	    if (mode == NORM)
		mode = LabelLayerCol;

	    txt = (char *) MyElement + sizeof(struct Text);
	    DrawVecTxt(RP,my.text->x_pos,my.text->y_pos,txt,my.text->Size,\
		my.text->Angle,mode);
	    break;

	case LABEL_ID:
	    size = my.rect->Thickness;
	    if (mode == NORM)
		mode = LabelLayerCol;

	    switch (my.element->Item)
	    {
		case LINE_ID:
		    DoLine(my.line->x_start, my.line->y_start, my.line->x_end,\
			   my.line->y_end, size, mode);
		    break;

		case ELLIPSE_ID:
		    DoEllipse(my.ellipse->x_pos, my.ellipse->y_pos, my.ellipse->size_x,\
			      my.ellipse->size_y, size, mode);
		    break;

		case RECT_ID:
		    DoRect(my.rect->x_pos, my.rect->y_pos, my.rect->x1_pos,\
			   my.rect->y1_pos, size, mode);
		    break;
	    }
	    break;
    }
}

void SelElement(struct Element *element)
{
    DrawEntry(element,SELECT);
    SelectElement = element;
    SelectFlag = TRUE;
    SelectX = x;
    SelectY = y;
}

void DoSelect(void)
{
    union elements my;

    HideIt();

    if (x != SelectX AND y != SelectY)
    {
	if (SelectFlag)
	    DrawEntry(SelectElement,NORM);
	SelectFlag = FALSE;
    }
    if (SelectFlag == FALSE)
	my.element = (struct Element *) PCBStart;
    else
    {
	DrawEntry(SelectElement,NORM);
	my.element = (struct Element *) GetNextEntry(SelectElement);
    }
    while (my.element->Type != 0)
    {
	switch (my.element->Type)
	{
	    case PAD_ID:
		if (my.pad->x_pos == x AND my.pad->y_pos == y)
		{
		    SelElement(my.element);
		    return;
		}
		break;
	    case TRACE_ID:
		if (((my.trace->x_start == x AND my.trace->y_start == y) \
			OR (my.trace->x_end == x AND my.trace->y_end == y)) \
			AND my.trace->Layer == ActiveLayer)
		{
		    SelElement(my.element);
		    return;
		}
		break;
	    case TEXT_ID:
		if (my.text->x_pos == x AND my.text->y_pos == y AND my.text->Layer==ActiveLayer)
		{
		    SelElement(my.element);
		    return;
		}
		break;
	    case LABEL_ID:
		switch (my.element->Item)
		{
		    case LINE_ID:
			if ((my.line->x_start == x AND my.line->y_start == y) \
				OR (my.line->x_end == x AND my.line->y_end == y))
			{
			    SelElement(my.element);
			    return;
			}
			break;
		    case RECT_ID:
			if ((my.rect->x_pos == x AND my.rect->y_pos == y) \
				OR (my.rect->x1_pos == x AND my.rect->y1_pos == y))
			{
			    SelElement(my.element);
			    return;
			}
			break;
		    case ELLIPSE_ID:
			if (my.ellipse->x_pos == x AND my.ellipse->y_pos == y)
			{
			    SelElement(my.element);
			    return;
			}
			break;
		}
		break;
	}
	my.element = GetNextEntry(my.element);
    }
    SelectFlag = FALSE;
    ShowIt();
}

int Delete(void)
{
    if (SelectFlag == TRUE)
    {
	HideIt();
	DrawEntry(SelectElement,CLEAR);
	DelEntry(SelectElement);
	SelectFlag = FALSE;
	ShowIt();
	return (1);
    }
}

short MoveFlag, move_x, move_y;

void MoveIt(void)
{
    struct Pad *MyPad;
    struct Trace *MyTrace;
    union elements my;

    if (SelectFlag)
    {
	my.element = SelectElement;
	ModCnt++;
	switch (SelectElement->Type)
	{
	    case PAD_ID:
		my.pad->x_pos = x;
		my.pad->y_pos = y;
		break;

	    case TRACE_ID:
		if ((my.trace->x_start == move_x) AND (my.trace->y_start == move_y))
		{
		    my.trace->x_start = x;
		    my.trace->y_start = y;
		}
		else
		{
		    my.trace->x_end = x;
		    my.trace->y_end = y;
		}
		break;

	    case TEXT_ID:
		my.text->x_pos = x;
		my.text->y_pos = y;
		break;

	    case LABEL_ID:
		switch (my.element->Item)
		{
		    case LINE_ID:
			if (my.line->x_start == move_x AND my.line->y_start == move_y)
			{
			    my.line->x_start = x;
			    my.line->y_start = y;
			}
			else
			{
			    my.line->x_end = x;
			    my.line->y_end = y;
			}
			break;


		    case RECT_ID:
			if (my.rect->x_pos == move_x AND my.rect->y_pos == move_y)
			{
			    my.rect->x_pos = x;
			    my.rect->y_pos = y;
			}
			else
			{
			    my.rect->x1_pos = x;
			    my.rect->y1_pos = y;
			}
			break;

		    case ELLIPSE_ID:
			my.ellipse->x_pos = x;
			my.ellipse->y_pos = y;
			break;
		}
		break;
	}
	SelectFlag = FALSE;
    }
    else {
	while (MyPad=FindPad(move_x,move_y))
	{
	    MyPad->x_pos = x;
	    MyPad->y_pos = y;
	    ModCnt++;
	}
	while (MyTrace=FindTrPntLayer(move_x,move_y,ActiveLayer))
	{
	    if ((MyTrace->x_start == move_x) AND (MyTrace->y_start == move_y))
	    {
		MyTrace->x_start = x;
		MyTrace->y_start = y;
		ModCnt++;
	    }
	    else
	    {
		MyTrace->x_end = x;
		MyTrace->y_end = y;
		ModCnt++;
	    }
	}
    }
    DoRefresh();
    DelFlag = FALSE;
    DrawKey = 0;
    MoveFlag = FALSE;
}

short tangle = 0;
short tx,ty;

void Draw45Trace(short mode)
{
    if (tangle == 45)
	Draw45(mode);
    else
	DrawTrace(x1,y1,x2,y2,TraceSize,mode);
}


void Draw45(short mode)
{
    if (ABS(x2-x1) < ABS(y2-y1))
    {
	if (y1 > y2)
	{
	    ty = y1 - (ABS(y2 - y1) - ABS(x2 - x1));
	    tx = x1;
	}
	else
	{
	    ty = y1 + ABS(y2 - y1) - ABS(x2 - x1);
	    tx = x1;
	}
    }
    else
    {
	if (x1 > x2)
	{
	    ty = y1;
	    tx = x1-(ABS(x2-x1)-ABS(y2-y1));
	}
	else
	{
	    ty = y1;
	    tx = x1+ABS(x2-x1)-ABS(y2-y1);
	}
    }
    DrawTrace(x1,y1,tx,ty,TraceSize,mode);
    DrawTrace(tx,ty,x2,y2,TraceSize,mode);
}

void SetTrace(void)
{
    if (!ActiveLayer) return;

    if ((Message->Class)==MOUSEBUTTONS)
    {
	if (Message->Code==SELECTDOWN)
	{
	    if (DrawKey)
	    {
		if (x1==x AND y1==y)
		{
		    DrawKey = 0;
		    SetDrMd(RP,JAM1);
		}
		else
		{
		    HideIt();

		    if (tangle == 45)
		    {
			Draw45(ActiveLayer);
			AddTrace(TraceSize, x1, y1, tx, ty, ActiveLayer);
			AddTrace(TraceSize, tx, ty, x2, y2, ActiveLayer);
		    }
		    else
		    {
			DrawTrace(x1, y1, x2, y2, TraceSize, ActiveLayer);
			AddTrace(TraceSize, x1, y1, x2, y2, ActiveLayer);
		    }
		    x1 = x2;
		    y1 = y2;
		    ShowIt();
		}
	    }
	    else
	    {
		x1 = x2 = x;
		y1 = y2 = y;
		SetDrMd(RP,JAM1|COMPLEMENT);
		DrawKey = 1;
	    }
	}
    }
    if (Message->Class==MOUSEMOVE AND (x2!=x OR y2!=y))
    {
	SetDrMd(RP,JAM1|COMPLEMENT);
	if (DrawKey)
	{
	    if (tangle != 90)
	    {
		Draw45Trace(INVERS);
		x2 = x;
		y2 = y;
		Draw45Trace(INVERS);
	    }
	    else
	    {
		DrawTrace(x1,y1,x2,y2,TraceSize,INVERS);
		if (ABS(x-x1) < ABS(y-y1))
		{
		    x2 = x1;
		    y2 = y;
		}
		else
		{
		    x2 = x;
		    y2 = y1;
		}
		DrawTrace(x1,y1,x2,y2,TraceSize,INVERS);
	    }
	}
    }
}

void SetPad(void)
{
    if (Message->Class == MOUSEBUTTONS)
    {
	if (Message->Code==SELECTDOWN)
	{
	    if (FindPad(x2, y2))
		DelPad(x2,y2);
	    HideIt();
	    DrawPad(x2, y2, PinSize, PadCol);
	    AddPad(PinSize, x2, y2);
	    x2 = x;
	    y2 = y;
	    ShowIt();
	}
    }
    if (Message->Class==MOUSEMOVE)
    {
	if (x!=x2 OR y!=y2)
	{
	    DrawPad(x2,y2,PinSize,INVERS);
	    x2 = x;
	    y2 = y;
	    DrawPad(x2,y2,PinSize,INVERS);
	}
    }
}
