/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     part.c
 * CONTENTS: Routines for selecting Parts
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "debug.h"
#include "/obj/prog-protos.h"

Prototype void SelectPart(void);
Prototype void CleanUpAddPart(void);
Prototype void SetPart(void);
Prototype int CopyPart2PCB(void);
Prototype void InitPartAdd(void);
Prototype void HidePart(void);

Prototype struct Element *PartStart, *PartEnd, *PartMax;

struct Element *PartStart, *PartEnd, *PartMax;

UBYTE *MaskPlane;
struct BitMap *MaskBitMap;

struct Screen *PartScr, *PartSaveScr;
struct Window *PartWin, *PartSaveWin;
struct TmpRas PartRas;
PLANEPTR PartPlane;
short part_x, part_y;
short part_size_x_in, part_size_y_in;

struct BitMap *SetupMyBitMap(short width, short height, short depth)
{
    struct BitMap *mybm;
    short i;

    if (!(mybm = (struct BitMap *) AllocMem(sizeof (struct BitMap), MEMF_CHIP|MEMF_CLEAR))) {
	rtEZRequest("SetupMyBitMap():\n Not enough chip memory for Bitmap struct!"," Forget it ",NULL,(struct TagItem *) mtags);
	return(NULL);
    }

    InitBitMap(mybm, depth, width, height);

    for (i=0; i<depth; i++)
	if (!(mybm->Planes[i] = AllocRaster(mybm->BytesPerRow*8, mybm->Rows))) {
	    rtEZRequest("SetupMyBitMap(): Not enough chip memory for Bitplane %d free!"," Forget it ",NULL,(struct TagItem *) mtags,i);
	    return(NULL);
	}
    return (mybm);
}

void FreeMyBitMap(struct BitMap *mybm)
{
    short i = 0;

    while (mybm->Planes[i])
	FreeRaster(mybm->Planes[i++], mybm->BytesPerRow*8, mybm->Rows);

    FreeMem(mybm, sizeof (struct BitMap)); // BitMap-Structure lschen
}

void BltToSave(short x, short y)
{
    if (!PartSaveWin) return;

    x = Zoll2X(x);
    y = Zoll2Y(y);
    SafeSetWriteMask(PartSaveWin->RPort,255);

    BltBitMap(RP->BitMap,x,y,PartSaveWin->RPort->BitMap,0,0,part_x,part_y,0xC0,0xFF,NULL);
}

void BltFromSave(short x, short y)
{
    if (!PartSaveWin) return;

    x = Zoll2X(x);
    y = Zoll2Y(y);
    SafeSetWriteMask(RP,255);

    BltBitMapRastPort(PartSaveWin->RPort->BitMap,0,0,RP,x,y,part_x,part_y,0xC0);
}

void BltFromPart(short x, short y)
{
    if (!PartWin) return;

    x = Zoll2X(x);
    y = Zoll2Y(y);
    SafeSetWriteMask(PartWin->RPort,255);
    SafeSetWriteMask(RP,255);

    BltMaskBitMapRastPort(PartWin->RPort->BitMap,0,0,RP,x,y,part_x,part_y,0xE0,MaskPlane);
}

void HidePart(void)
{
    if (Tool == PART_ADD && !HideFlag)
	BltFromSave(x1,y1);

    if (MaskBitMap)
    {
	FreeMyBitMap(MaskBitMap);
	MaskBitMap = NULL;
    }

    if (PartPlane)
    {
	FreeRaster(PartPlane, part_x, part_y);
	PartPlane = NULL;
    }

    if (PartWin)
    {
	CloseWindow(PartWin);
	PartWin = NULL;
    }

    if (PartScr)
    {
	CloseScreen(PartScr);
	PartScr = NULL;
    }

    if (PartSaveWin)
    {
	CloseWindow(PartSaveWin);
	PartSaveWin = NULL;
    }

    if (PartSaveScr)
    {
	CloseScreen(PartSaveScr);
	PartSaveScr = NULL;
    }
}

void CleanUpAddPart(void)
{
    HidePart();

    Tool = PART_COPY;
    DoOnXY();
    PartEnd = PartStart;
    part_size_x_in = part_size_y_in = 0;
}

int InitPartWin(short width, short height, short depth)
{
    if (!(PartScr = OpenScreenTags(NULL, SA_Width, width, SA_Height, height, SA_Depth, depth, SA_Behind, (long) TRUE, TAG_END))) {
	rtEZRequest("Unable to open PartScr !!!\nMaybe you ran out of chip-memory","  OK  ",NULL,NULL);
	return(0);
    }
    if (!(PartWin = OpenWindowTags(NULL, WA_CustomScreen, PartScr, WA_Width, width, WA_Height, height, TAG_END))) {
	rtEZRequest("Unable to open PartWin !!!\nMaybe you ran out of chip-memory","  OK  ",NULL,NULL);
	return(0);
    }
    if (!(PartSaveScr = OpenScreenTags(NULL, SA_Width, width, SA_Height, height, SA_Depth, depth, SA_Behind, (long) TRUE, TAG_END))) {
	rtEZRequest("Unable to open PartSaveScr !!!\nMaybe you ran out of chip-memory","  OK  ",NULL,NULL);
	return(0);
    }
    if (!(PartSaveWin = OpenWindowTags(NULL, WA_CustomScreen, PartSaveScr, WA_Width, width, WA_Height, height, TAG_END))) {
	rtEZRequest("Unable to open PartSaveWin !!!\nMaybe you ran out of chip-memory","  OK  ",NULL,NULL);
	return(0);
    }
    if (!(PartPlane = AllocRaster(width,height))) {
	rtEZRequest("Unable to open PartPlane !!!\nMaybe you ran out of chip-memory","  OK  ",NULL,NULL);
	return(0);
    }
    InitTmpRas(&PartRas,(char *) PartPlane,RASSIZE(width,height));
    PartWin->RPort->TmpRas = &PartRas;

    if (!(MaskBitMap = SetupMyBitMap(width, height, 1)))
	return(0);

    MaskPlane = MaskBitMap->Planes[0];

    return(1);
}

void InitPartAdd(void)
{
    struct RastPort *PRp, *SaveRp;
    struct Pad *SaveStart, *SaveEnd;
    int    i, pl, Anzahl;
    struct BitMap *mybm;
    UBYTE  *Plane;
    short  SaveStatusHeight;

    short SaveOffX, SaveOffY;

    UpdateItem(SelItem,UNSELECTED,-1);
    UpdateItem(3,SELECTED,-1);
    SelItem = 3;

    if (part_size_x_in == 0)
    {
	part_size_x_in = x2 - x1;
	part_size_y_in = y2 - y1;
    }
    part_x = Zoll2PixX(part_size_x_in);
    part_y = Zoll2PixY(part_size_y_in);

    part_x +=16;
    part_y +=16;

    part_x &= 0xFFF0;
    part_y &= 0xFFF0;

    if (!(InitPartWin(part_x,part_y,ScrDepth))) {
	puts("Can't open InitPartWin");
	return;
    }

    PRp = PartWin->RPort;

    SetDrMd(PRp,JAM1);
    SetAPen(PRp,ClsCol);
    RectFill(PRp,0,0,part_x,part_y);

    SaveRp = RP;
    SaveStart = PCBStart;
    SaveEnd = PCBEnd;
    SaveOffX = PCBOFFSETX;
    SaveOffY = PCBOFFSETY;

    RP = PRp;
    PCBStart = (struct Pad *) PartStart;
    PCBEnd = (struct Pad *) PartEnd;
    SaveStatusHeight = StatusHeight;
    StatusHeight = 0;
    PCBOFFSETX = 0;
    PCBOFFSETY = 0;

    RefreshScreen();

    StatusHeight = SaveStatusHeight;

    RP = SaveRp;
    PCBStart = SaveStart;
    PCBEnd = SaveEnd;

    PCBOFFSETX = SaveOffX;
    PCBOFFSETY = SaveOffY;

    mybm = PartWin->RPort->BitMap;

    Anzahl = (mybm->BytesPerRow) * (mybm->Rows);

    Plane = mybm->Planes[0];

    for (i=0; i<Anzahl; i++)
	MaskPlane[i] = ~Plane[i];   // MaskPlane = NOT(Plane 0)

    for (pl=1; pl<ScrDepth; pl++) {
	Plane = mybm->Planes[pl];

	for (i=0; i<Anzahl; i++)
	    MaskPlane[i] |= Plane[i];	// Bitplanes werden ver'OR'd
    }
    Tool = PART_ADD;

    x1 = x;
    y1 = y;

    BltToSave(x1, y1);
    BltFromPart(x1, y1);
}

void AddToKoord(struct Element *element, short offx, short offy)
{
    union elements my;

    if (element->Type == 0)
	return;

    my.element = element;

    switch (my.element->Type)
    {
	case PAD_ID:
	    my.pad->x_pos += offx;
	    my.pad->y_pos += offy;
	    break;

	case TRACE_ID:
	    my.trace->x_start += offx;
	    my.trace->y_start += offy;
	    my.trace->x_end += offx;
	    my.trace->y_end += offy;
	    break;

	case TEXT_ID:
	    my.text->x_pos += offx;
	    my.text->y_pos += offy;
	    break;

	case LABEL_ID:
	    switch (my.element->Item)
	    {
		case LINE_ID:
		    my.line->x_start += offx;
		    my.line->y_start += offy;
		    my.line->x_end   += offx;
		    my.line->y_end   += offy;
		    break;

		case ELLIPSE_ID:
		    my.ellipse->x_pos += offx;
		    my.ellipse->y_pos += offy;
		    break;

		case RECT_ID:
		    my.rect->x_pos += offx;
		    my.rect->y_pos += offy;
		    my.rect->x1_pos += offx;
		    my.rect->y1_pos += offy;
		    break;
	    }
	    break;
    }
}

int Copy2Part(struct Element *myel)
{
    long size;

    size = GetEntrySize(myel);

    if (((char *) PartEnd + size) > (char *) PartMax)
	return(0);

    CopyEntry(myel, (APTR) PartEnd);
    AddToKoord(PartEnd,-x1,-y1);

    (char *) PartEnd += size;

    PartEnd->Type = 0;
}

int CopyPart2PCB(void)
{
    long size;
    struct Element *myel;

    size = (char *) PartEnd - PartStart;

    if (((char *) PCBEnd + size) > (char *) PCBMax)
	return(0);

    UpdateUndo();

    CopyMem((char *) PartStart, (char *) PCBEnd, size);

    myel = PCBEnd;
    (char *) PCBEnd += size;
    PCBEnd->Type = 0;

    do {
	AddToKoord(myel,x2,y2);
	ModCnt++;
    } while (myel = GetNextEntry(myel));

    return(1);
}

void SetPart(void)
{
    if ((Message->Class == MOUSEBUTTONS) && (Message->Code == SELECTDOWN))
    {
	x2 = x;
	y2 = y;
	CopyPart2PCB();
	BltToSave(x1, y1);
    }
    else if (Message->Class==MOUSEMOVE)
    {
	BltFromSave(x1, y1);
	x1 = x;
	y1 = y;
	BltToSave(x1, y1);
	BltFromPart(x1, y1);
	WaitTOF();
    }
}

void SelectPart(void)
{
    short du_x,du_y,du_x1,du_y1;
    short tflag;
    union elements my;

    if (Message->Class == MOUSEBUTTONS)
    {
	if (Message->Code==SELECTDOWN && DrawKey==0)
	{
	    x1=x2=x;
	    y1=y2=y;
	    DrawKey = 1;
	    OffXY();
	}
	else if (Message->Code==SELECTDOWN && DrawKey==1)
	{
	    if (x1 > x2)	// Sortieren der Rechteckkoordinaten
	    {
		du_x = x2;	// so da x2>x1 und y2>y1
		x2 = x1;	// x1,y1 enthalt obere, linke Koordinate
		x1 = du_x;	// x2,y2 untere, rechte Koordinate
	    }
	    if (y1 > y2)
	    {
		du_y = y2;
		y2 = y1;
		y1 = du_y;
	    }

	    if ((Tool != PART_CLEAR) || (rtEZRequest("Hit OK to clear Rect ?","  OK  | Cancel ",NULL,NULL) == 1))
	    {
		if (Tool == PART_COPY)
		{
		    PartEnd = PartStart;
		    PartEnd->Item = 0;
		}
		UpdateUndo();

		my.element = (struct Element *) PCBStart;
		while (my.element && (my.element->Type != 0))
		{
		    tflag = 0;
		    switch (my.element->Type)
		    {
			case PAD_ID:
			    du_x = my.pad->x_pos;
			    du_y = my.pad->y_pos;
			    tflag = 1;
			    break;
			case TRACE_ID:
			    du_x = my.trace->x_start;
			    du_y = my.trace->y_start;
			    du_x1 = my.trace->x_end;
			    du_y1 = my.trace->y_end;
			    tflag = 2;
			    break;

			case TEXT_ID:
			    du_x = my.text->x_pos;
			    du_y = my.text->y_pos;
			    tflag = 1;
			    break;

			case LABEL_ID:
			    switch (my.element->Item)
			    {
				case LINE_ID:
				    du_x = my.line->x_start;
				    du_y = my.line->y_start;
				    du_x1 = my.line->x_end;
				    du_y1 = my.line->y_end;
				    tflag = 2;
				    break;

				case ELLIPSE_ID:
				    du_x = my.ellipse->x_pos;
				    du_y = my.ellipse->y_pos;
				    tflag = 1;
				    break;

				case RECT_ID:
				    du_x = my.rect->x_pos;
				    du_y = my.rect->y_pos;
				    du_x1 = my.rect->x1_pos;
				    du_y1 = my.rect->y1_pos;
				    tflag = 2;
				    break;
			    }
			    break;
		    }
		    if (tflag > 0)
		    {
			if (du_x<x2 && du_x>x1 && du_y<y2 && du_y>y1)
			{
			    if (tflag == 1)
			    {
				if (Tool == PART_CLEAR)
				    DelEntry(my.element);

				else if (Tool == PART_COPY)
				{
				    Copy2Part(my.element);
				    tflag = 0;
				}
				else if (Tool == PART_CUT)
				{
				    Copy2Part(my.element);
				    DelEntry(my.element);
				    tflag = 0;
				}
			    }
			    else if (du_x1<x2 && du_x1>x1 && du_y1<y2 && du_y1>y1 && tflag==2)
			    {
				if (Tool == PART_CLEAR)
				    DelEntry(my.element);

				else if (Tool == PART_COPY)
				{
				    Copy2Part(my.element);
				    tflag = 0;
				}
				else if (Tool == PART_CUT)
				{
				    Copy2Part(my.element);
				    DelEntry(my.element);
				    tflag = 0;
				}
			    }
			    else tflag = 0;
			}
			else tflag = 0;
		    }
		    if (tflag == 0)
			my.element = (struct Element *) GetNextEntry(my.element);
		}
		DrawKey = 0;

		if (Tool == PART_CLEAR)
		    DoRefresh();
		else if (Tool == PART_COPY)
		{
		    DoRect(x1,y1,x2,y2,0,INVERS);
		    InitPartAdd();
		}
	    }
	    else
		 DoRect(x1,y1,x2,y2,0,INVERS);

	    DrawKey = 0;
	    // DoRefresh();
	    if (Tool == PART_CLEAR)
		OnXY();
	}
    }
    if (Message->Class==MOUSEMOVE && DrawKey)
    {
	DoRect(x1,y1,x2,y2,0,INVERS);
	x2 = x;
	y2 = y;
	DoRect(x1,y1,x2,y2,0,INVERS);
    }
}
