/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     solder.c
 * CONTENTS: draw routines for pads & traces
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "debug.h"
#include "/obj/prog-protos.h"

Prototype USHORT *InstallChip(USHORT *FastImg,long size);
Prototype void PrintText(struct RastPort *Rp,short tx,short ty,char *text, short col,short dmode);
Prototype void PrintKoordsXY(void);
Prototype void PRKoords(void);
Prototype void EllipsePaint(struct RastPort *rp,short x,short y,short rh,short rv);
Prototype void DreieckPaint(struct RastPort *rp, x1,y1,x2,y2,x3,y3,x4,y4);
Prototype void DrawLine(short x,short y,short x1,short y1);
Prototype void DrawTrace(short xs,short ys,short xe,short ye,short size,short layer);
Prototype void DrawPad(short xd,short yd,short size,short color);
Prototype void SetLayerColor(short layer);

// Prototype SafeSetWriteMask(struct RastPort *RP, int CompoMask);

Prototype short ScrLimXstart, ScrLimYstart, ScrLimXend, ScrLimYend;

short ScrLimXstart, ScrLimYstart, ScrLimXend, ScrLimYend;

USHORT *InstallChip(USHORT *FastImg,long size)
{
    USHORT *chipimg;

    if (!(chipimg = (USHORT *) AllocMem(size,MEMF_CHIP)))
	Ende(puts("ERROR: Nicht genug chip ram frei (InstallImg)"));
    CopyMem((char *) FastImg,(char *) chipimg,size);
    return(chipimg);
}

void PrintText(Rp,tx,ty,text,col,dmode)
struct RastPort *Rp;
short tx,ty;
char *text;
short col,dmode;
{
    SetDrMd(Rp,dmode);
    SetAPen(Rp,col);
    Move(Rp, tx, ty);
    Text(Rp, text, strlen(text));
}

void PrintKoordsXY(void)
{
    static char txt[40];
    static char d1[40], d2[40];
    short len;

    long lix, liy, xvs, xns, yvs, yns;	// ns = NachkommaStelle vs = Vorkommastelle
    char  vx, vy;			// vx = Vorzeichen x

/*
    lix = ABS(x);
    liy = ABS(y);

    xvs = lix/1600;
    yvs = liy/1600;
    xns = lix-(lix*1600);
    yns = liy-(liy*1600);

    if (x<0) vx = '-'; else vx = ' ';
    if (y<0) vy = '-'; else vy = ' ';

    if (x<0 OR x>PCBSizeX OR y<0 OR y>PCBSizeY)
	sprintf(txt," (outside) x= %c%d.%04din  y= %c%d.%04din ",vx,xvs,xns,vy,yvs,yns);
    else
	sprintf(txt,"           x= %c%d.%04din  y= %c%d.%04din ",vx,xvs,xns,vy,yvs,yns);
*/

    lix = ABS((long) x * 15875);	    // /1000000
    xvs = lix / 1000000;
    xns = (lix - (xvs * 1000000)) / 10000;

    liy = ABS((long) y * 15875);	    // /1000000
    yvs = liy / 1000000;
    yns = (liy - (yvs * 1000000)) / 10000;

    if (x<0) vx = '-'; else vx = ' ';
    if (y<0) vy = '-'; else vy = ' ';

    if (x<0 OR x>PCBSizeX OR y<0 OR y>PCBSizeY)
	sprintf(txt," (outside) x= %c%d.%02ldmm  y= %c%d.%02ldmm ",vx,xvs,xns,vy,yvs,yns);
    else
	sprintf(txt,"           x= %c%d.%02ldmm  y= %c%d.%02ldmm ",vx,xvs,xns,vy,yvs,yns);

    len = TextLength(RPs,txt,strlen(txt));
    PrintText(RPs,MaxScreenX-len-8,fontheight-1,txt,1,JAM2);

/*
    InToStr(x,d1);
    InToStr(y,d2);

    if (x<0 OR x>PCBSizeX OR y<0 OR y>PCBSizeY)
	sprintf(txt," (outside) x= %s  y= %s ",d1,d2);
    else
	sprintf(txt,"           x= %s  y= %s ",d1,d2);

    SetBPen(RPs,0);

    len = TextLength(RPs,txt,strlen(txt));
    PrintText(RPs,MaxScreenX-len-8,fontheight-1,txt,1,JAM2);
*/
}

void PRKoords(void)
{
    char tmp[40];

    strcpy(tmp,"CrazyPads (");
    strcat(tmp,versiontxt);
    strcat(tmp,"): ");

    strcat(tmp,&FRName[0]);
    tmp[40] = 0;

    SetAPen(RPs,0);
    SetDrMd(RPs,JAM1);
    RectFill(RPs,0,0,MaxScreenX,fontheight+3);
    SetAPen(RPs,2);
    Move(RPs,0,0); Draw(RPs,MaxScreenX,0);

    PrintText(RPs,3,fontheight-1,tmp,1, JAM1);

    PrintKoordsXY();
}


void EllipsePaint(struct RastPort *rp,short x,short y,short rh,short rv)
{
    struct AreaInfo AInfo;
    WORD	    Buffer[10];
    // short	       a,b,cx,cy;

    // if (PrtDraw == 0) {
	InitArea(&AInfo,Buffer,2);
	rp->AreaInfo = &AInfo;
	AreaEllipse(rp,x,y,rh,rv);
	AreaEnd(rp);
	return;
/*
    }
    else
    {
	a = rh + rv;
	cx = cy = 0;

	for (b=0; b<=a; b++)
	{
	    DrawEllipse(rp,x,y,cx,cy);
	    if (cx < rh) cx++;
	    else if (cy < rv) cy++;
	}
    }
*/
}

void DreieckPaint(struct RastPort *rp, x1,y1,x2,y2,x3,y3,x4,y4)
{
    struct AreaInfo AInfo;
    WORD Buffer[25];

    InitArea(&AInfo,Buffer,5);
    rp->AreaInfo = &AInfo;
    AreaMove(rp,x1,y1);
    AreaDraw(rp,x2,y2);
    AreaDraw(rp,x3,y3);
    if (x4 != -1)
	AreaDraw(rp,x4,y4);

    AreaEnd(rp);
    return;
}

void DrawLine(short x,short y,short x1,short y1)
{
    // if (PrtDraw == 0) {
	Move(RP,x,y);
	Draw(RP,x1,y1);
/*
    }
    else
	FLine(RP,x,y,x1,y1);
*/
}

void SetLayerColor(short layer)
{
    SetDrMd(RP,JAM1);

    if (PrtDraw == 1)
    {
	SetAPen(RP,1);
	return;
    }

    if (layer == LABELLAYER)
    {
	if (LabelLayer == LAYER_ACTIVE)
	    SetAPen(RP,LabelLayerCol);
    }
    else if ((ComponentLayer == LAYER_ACTIVE) && (SolderLayer == LAYER_ACTIVE))
    {
	if (layer & CP_COMPLETE)
	    layer -= CP_COMPLETE;
	else
	{
	    switch (layer)
	    {
		case COMPONENTLAYER:
		    SafeSetWriteMask(RP,CompoMask);
		    break;
		case SOLDERLAYER:
		    SafeSetWriteMask(RP,SolderMask);
		    break;
	     }
	}

	switch (layer)
	{
	    case COMPONENTLAYER:
		// SafeSetWriteMask(RP,CompoMask);
		SetAPen(RP,CompoLayerCol);
		break;
	    case SOLDERLAYER:
		// SafeSetWriteMask(RP,SolderMask);
		SetAPen(RP,SolderLayerCol);
		break;
	    case 0:
		SetAPen(RP,ClsCol);
		break;
	}
    }
    else
    {
	switch (layer)
	{
	    case COMPONENTLAYER:
		switch (ComponentLayer)
		{
		    case LAYER_ACTIVE:
		    SetAPen(RP,CompoLayerCol);
		    break;
		case LAYER_INACTIVE:
		    SetAPen(RP,CompoInactiveCol);
		    break;
		}
		break;
	    case SOLDERLAYER:
		switch (SolderLayer)
		{
		    case LAYER_ACTIVE:
			SetAPen(RP,SolderLayerCol);
			break;
		    case LAYER_INACTIVE:
			SetAPen(RP,SolderInactiveCol);
			break;
		}
		break;
	    case 0:
		SetAPen(RP,ClsCol);
		break;
	}
    }
}

void DrawTrace(short xs,short ys,short xe,short ye,short size,short layer)
{
    short a,b,Xe,Ye, Xs, Ys;
    short dx,dy;
    float m;	// Geradensteigung

    if (layer==COMPONENTLAYER && ComponentLayer==LAYER_HIDDEN)
	return;
    if (layer==SOLDERLAYER && SolderLayer==LAYER_HIDDEN)
	return;

#ifdef DEBUG
    if ((DebugLevel > 2) && (layer != INVERS))
    {
	printf("DrawTrace: xs=%d ys=%d xe=%d ye=%d (inch)\n",xs,ys,xe,ye);
	printf("DrawTrace: size=%d (inch)\n",size);
	printf("DrawTrace: layer=%d\n",layer);
    }
#endif

    xs = Zoll2X(xs);
    ys = Zoll2Y(ys);
    xe = Zoll2X(xe);
    ye = Zoll2Y(ye);

    Xe = ScrLimXend;
    Ye = ScrLimYend;
    Xs = ScrLimXstart;
    Ys = ScrLimYstart;

    if ((xs<Xs || xs>Xe || ys<Ys || ys>Ye) && (xe<Xs || xe>Xe || ye<Ys || ye>Ye))
	return;

    if (PrtDraw == TRUE)		 // fuer Druckausgabe
    {
	size -= DotSize;
	if (size<0) size = 1;
    }
    else size -= (1600/ActualdpiY);	 // 1000dpi/Actualdpi

    size /= 2;

    if (xs-xe != 0)
    {
	m = (float) (ys-ye)*2 / (xs-xe);
	a = (short) ((float) m * 10);

	// ACHTUNG: Die Steigung entspricht NICHT der Standartsteigung,
	//	    da das Koordinatensystem der y-Achse umgekehrt ist !
	//---------------------------------------------------
	//  |	    |	    |	    |	    |	    |	    |
	// -+---> X |  O    |	 O  |	    | O     |  O    |
	//  |	    |  |    |	/   | O--O  |  \    |  |    |
	//  |	    |  |    |  /    |	    |	\   |  |    |
	//  \/ Y    |  O    | O     |	    |	 O  |  O    |
	//---------------------------------------------------
	//	      m=-oo   m= -1   m= 0    m=  1   m= +oo oder -oo
	//
	//	      m<-2  -2<m<-.5 -.5<m<.5 .5<m<2  m>2
	//
	//	     a<-20  -20<a<-5 -5<a<5   5<a<20  a>20

	if (a > 21)		// Steigung 2.0 bis + unendlich
	{
	    dx = Zoll2PixX(size);
	    dy = 0;
	}
	else if (a > 5) 	// Steigung 0.5 bis 2.0
	{
	    b = (short) (((long) size * 10000) / 1414);
	    if (b > 10)
		b /= 10;
	    else if (b > 4) b = 1;
	    else b = 0;

	    dx = -Zoll2PixX(b);
	    dy = -Zoll2PixY(b);
	}
	else if (a > -4)	// Steigung -0.5 bis 0.5
	{
	    dx = 0;
	    dy = Zoll2PixY(size);
	}
	else if (a > -21)	// Steigung -2.0 bis -0.5
	{
	    b = (short) (((long) size * 10000) / 1414);
	    if (b > 10)
		b /= 10;
	    else if (b > 4) b=1;
	    else b = 0;

	    dx = -Zoll2PixX(b);
	    dy = Zoll2PixY(b);
	}
	else if (a < -20)      // Steigung 2.0 bis + unendlich
	{
	    dx = Zoll2PixX(size);
	    dy = 0;
	}
    }
    else
    {
	dx = Zoll2PixX(size);
	dy = 0;
    }

#ifdef DEBUG
    if ((DebugLevel > 2) && (layer != INVERS))
    {
	printf("DrawTrace: xs=%d ys=%d xe=%d ye=%d\n",xs,ys,xe,ye);
	printf("DrawTrace: dx= %d  dy= %d\n",dx,dy);
	printf("DrawTrace: size= %d\n",size);
	printf("DrawTrace: steigung= %d\n\n",a);
    }
#endif

    if (layer == INVERS)
    {
	SetDrMd(RP,JAM1|COMPLEMENT);
	if (Zoll2PixX(size) < 2)
	{
	    DrawLine(xs,ys,xe,ye);
	}
	else {
	/*
	    DrawEllipse(RP,xs,ys,Zoll2PixX(size),Zoll2PixY(size));
	    DrawEllipse(RP,xe,ye,Zoll2PixX(size),Zoll2PixY(size));
	    DrawLine(xs+dx,ys-dy,xe+dx,ye-dy);
	    DrawLine(xs-dx,ys+dy,xe-dx,ye+dy);
	*/
	    Move(RP,xs+dx,ys-dy);
	    Draw(RP,xe+dx,ye-dy);
	    Draw(RP,xe-dx,ye+dy);
	    Draw(RP,xs-dx,ys+dy);
	    Draw(RP,xs+dx,ys-dy);
	}
    }
    else
    {
	if (layer == SELECT)
	{
	    SetDrMd(RP,JAM1);
	    SafeSetWriteMask(RP,SelectMask);
	    SetAPen(RP,SelectCol);
	}
	else if (PrtDraw == TRUE)
	{
	    SetDrMd(RP,JAM1);
	    SetAPen(RP,1);
	}
	else
	    SetLayerColor(layer);

	if (Zoll2PixX(size)<2 && dx==1 && dy==0)
	{
	    DrawLine(xs+1,ys,xe+1,ye);
	    DrawLine(xs,ys,xe,ye);
	}
	else
	{
	    // Die Routine DreieckPaint kann fehlerhafte Ergebnisse produzieren,
	    // Wenn Sie ueber einen Clippingbereich blitten muss.
	    // Ich habe im Moment keine Loesung fuer dieses Problem.
	    // Gilt fuer den alten 8371 Agnus !

	    DreieckPaint(RP,xs+dx,ys-dy,xe+dx,ye-dy,xe-dx,ye+dy,xs-dx,ys+dy);

/*
	    a = ABS(dx)+ABS(dy);

	    for (; a>=0; a--) {
		DrawLine(xs+dx,ys-dy,xe+dx,ye-dy);
		DrawLine(xe-dx,ye+dy,xs-dx,ys+dy);

		if (dx > 0) dx--;
		else if (dx < 0) dx++;
		else {
		    if (dy > 0) dy--;
		    else if (dy < 0) dy++;
		}
	    }
*/

	}
	if ((Zoll2PixX(size) > 1) && PrtDraw == TRUE)
	{
	    EllipsePaint(RP,xs,ys,Zoll2PixX(size)-1,Zoll2PixY(size)-1);
	    EllipsePaint(RP,xe,ye,Zoll2PixX(size)-1,Zoll2PixY(size)-1);
	}

	SafeSetWriteMask(RP,255);
/*
	SetAPen(RP,3);
	if (layer != INVERS)
	{
	    WritePixel(RP,xs,ys);
	    WritePixel(RP,xe,ye);
	}
*/
    }
}

void DrawPad(short xd,short yd,short size,short color)	   /* x,y, Gre, Farbe (-1=COMPLEMENTR, 0-7 die Farbe) */
{
    short pd,ph;

#ifdef DEBUG
    if (DebugLevel > 0)
	printf("DrawPad(): x=%d y=%d size=%d color=%d\n",xd,yd,size,color);
#endif

    xd=Zoll2X(xd);
    yd=Zoll2Y(yd);

#ifdef DEBUG
    if (DebugLevel > 0)
	printf("DrawPad(): x=%d y=%d\n\n",xd,yd);
#endif

    switch (size)
    {
	case 1:
	    pd = 30;
	    ph = 15;
	    break;

	case 2:
	    ph = StrToIn("0.8 mm")/2;
	    pd = StrToIn("1.5 mm")/2;
	    break;

	case 3:
	    ph = StrToIn("1 mm")/2;
	    pd = StrToIn("1.6 mm")/2;
	    break;

	case 4:
	    ph = StrToIn("1.3 mm")/2;
	    pd = StrToIn("2 mm")/2;
	    break;

	case 5:
	    pd = StrToIn("3.8 mm")/2;
	    ph = StrToIn("3.3 mm")/2;
	    break;

	default:
	    pd = 15*size;
	    ph = 7*size;
    }

    if (xd+pd<ScrLimXstart || xd-pd>ScrLimXend || yd+pd<ScrLimYstart || yd-pd>ScrLimYend)
	return;

/*
    if (size == 8 OR size == 9)
    {
	switch (color)
	{
	    case INVERS:
	    case SELECT:
		SetDrMd(RP,JAM1|COMPLEMENT);
		DrawEllipse(RP,xd,yd,Zoll2PixX(45),Zoll2PixY(45));
		DrawEllipse(RP,xd,yd,Zoll2PixX(21),Zoll2PixY(21));
		DrawLine(xd,yd-Zoll2PixY(45),xd,yd+Zoll2PixY(45));
		DrawLine(xd-Zoll2PixX(45),yd,xd+Zoll2PixX(45),yd);
		return;
		break;

	    case 0:
		SetDrMd(RP,JAM1);
		SetAPen(RP,ClsCol);
		EllipsePaint(RP,xd,yd,Zoll2PixX(45),Zoll2PixY(45));
		return;

	    default:
		DrawEllipse(RP,xd,yd,Zoll2PixX(45),Zoll2PixY(45));
		DrawEllipse(RP,xd,yd,Zoll2PixX(21),Zoll2PixY(21));
		DrawLine(xd,yd-Zoll2PixY(45),xd,yd+Zoll2PixY(45));
		DrawLine(xd-Zoll2PixX(45),yd,xd+Zoll2PixX(45),yd);

	    if (color != INVERS)
	    {
		Flood(RP,xd+Zoll2PixX(35),yd+Zoll2PixY(35),1);
		Flood(RP,xd-Zoll2PixX(35),yd-Zoll2PixY(35),1);
	    }
	    return;
	}
	else if (size == 10)
	{
	    DrawEllipse(RP,xd,yd,Zoll2PixX(45),Zoll2PixY(45));
	    DrawEllipse(RP,xd,yd,Zoll2PixX(21),Zoll2PixY(21));
	    DrawLine(xd,yd-Zoll2PixY(45),xd,yd+Zoll2PixY(45));
	    DrawLine(xd-Zoll2PixX(45),yd,xd+Zoll2PixX(45),yd);
	    if (color != INVERS)
	    {
		Flood(RP,xd+Zoll2PixX(31),yd-Zoll2PixY(31),1);
		Flood(RP,xd-Zoll2PixX(31),yd+Zoll2PixY(31),1);
	    }
	    return;
	}
    }
*/


#ifdef DEBUG
    if (DebugLevel > 0)
	printf("DrawPad(): real hole size: %d, real pad size\n",Zoll2PixX(ph),Zoll2PixY(pd));
#endif


    switch (color)
    {
	case INVERS:
	case SELECT:
	    SetDrMd(RP,JAM1|COMPLEMENT);
	    DrawEllipse(RP,xd,yd,Zoll2PixX(pd),Zoll2PixY(pd));
	    DrawEllipse(RP,xd,yd,Zoll2PixX(ph),Zoll2PixY(ph));
	    break;

	case 0:
	    SetDrMd(RP,JAM1);
	    SetAPen(RP,ClsCol);
	    EllipsePaint(RP,xd,yd,Zoll2PixX(pd),Zoll2PixY(pd));
	    break;

	default:
	    if (PrtDraw == TRUE)
	    {
		SetDrMd(RP,JAM1);
		SetAPen(RP,1);
	    }
	    else
	    {
		SetDrMd(RP,JAM1);
		SetAPen(RP,PadCol);
	    }

	    EllipsePaint(RP,xd,yd,Zoll2PixX(pd),Zoll2PixY(pd));

	    if (PrtDraw == 1)
		SetAPen(RP,0);
	    else
		SetAPen(RP,ClsCol);	// ClsCol = 1 !!!

	    EllipsePaint(RP,xd,yd,Zoll2PixX(ph),Zoll2PixY(ph));
	    break;
    }
}
