/********************************************************************
 *
 * PROGRAM:  CrazyPads
 * PART:     text.c
 * CONTENTS:
 *
 * (c) Copyright 1990-94 by Klaus Rotter. All rights reserved.
 * EMail: klaus@rotters.de
 *
 * Use it at your own risk. The program and the source code may be 
 * distributed under the terms of the GNU Public Licence (GPL).
 * For more informations read mainmodule (main.c) and CrazyPads.doc.
 *
 ********************************************************************/

#include "defs.h"
#include "debug.h"
#include "/obj/prog-protos.h"

Prototype void FontInit(void);
Prototype int DrawVecChar(struct RastPort *Rp, short x, short y, UBYTE z,short size,short angle,short layer);
Prototype int DrawVecTxt(struct RastPort *Rp, short x, short y, UBYTE *txt,short size,short angle,short layer);
Prototype void InitDrawTxt(void);
Prototype void SetText(void);
Prototype void DrawCursor(short x, short y);
Prototype void DrawDemoText(void);

Prototype char *FontMem;

#define ENDVEC -127
#define NEWVEC -126

struct VFChar
{
    UBYTE  Character, Fill;	    /* ASCII Code 0-255 */
    short AbsSizeX, AbsSizeY;	/* Absolute X und Y Gren */
    char  *VFCKoords;		    /* Zeiger auf Vektorkoordinaten */
};

short TextZoom = 1;
short TextMirror = 1;
short TextAngle;
short len_x, len_y;

struct VFChar MyVFChar[255-32];

char *FontMem = NULL;
int run, line;

void fnterr(char *str)
{
    printf("Syntax error in `crazypads.fnt' line %d\nERROR: %s\n", line, str);
//    rtEZRequest("Syntax error in `crazypads.fnt' line %d\nERROR: %s\n","Oh No",NULL,(struct TagItem *) mtags, line, str);
    Ende(0);
}

void CheckFntMemSize(void)
{
    if ((run+10) > CP_FONTMEM_SIZE)
	fnterr("Font memory exceeded.\nYou can ask a wizard to enlarge me\nI must quit");
}

void FontInit(void)
{
    FILE    *fh;
    int     code, xcord, ycord;
    BOOL    rtflag, defflag;
    long    size;
    char    *fbuf, *ptr, *nptr, *fbufmax;

#ifdef DEBUG
    if (DebugLevel > 0)
	printf("Enter FontInit()\n");
#endif

    if (!(fh = fopen("crazypads.fnt","r")))
    {
	rtEZRequest("Sorry, I can't open `crazypads.fnt'\n I must quit\n","Oh No",NULL,(struct TagItem *) mtags);
	Ende(0);
    }

    if (!(FontMem = (char *) malloc(CP_FONTMEM_SIZE)))
    {
	rtEZRequest("Sorry, I can't allocate memory for crazypads.fnt\n Maybe you ran out of memory\n I must quit\n","Oh No",NULL,(struct TagItem *) mtags);
	Ende(0);
    }

    fseek(fh, 0, SEEK_END);	    /*	seek to end	*/
    size = ftell(fh) + 2;	    /*	size of file	*/

    if (!(fbuf = (char *) malloc(size)))
    {
	rtEZRequest("Sorry, I can't allocate memory for font buffer\n Maybe you ran out of memory\n I must quit\n","Oh No",NULL,(struct TagItem *) mtags);
	Ende(0);
    }

    size -= 2;

#ifdef DEBUG
    if (DebugLevel > 0)
	printf("FontInit(): fbuf 0x%08x Size %d\n",fbuf,size);
#endif

    fseek(fh, 0, SEEK_SET);	   /*  seek to Start  */

    if (fread(fbuf, 1, size, fh) != size)
    {
	rtEZRequest("Sorry, I can't read crazypads.fnt correctly\n Maybe you ran out of memory\n I must quit\n","Oh No",NULL,(struct TagItem *) mtags);
	Ende(0);
    }
    fbufmax = fbuf + size;

    ptr = fbuf;
    while (ptr < fbufmax)
    {
	if (*ptr == '\t')
	    *ptr = ' ';
	ptr++;
    }

#ifdef DEBUG
    if (DebugLevel > 0)
	printf("FontInit(): FontMem at 0x%08x\n",FontMem);
#endif

    line = 1;
    run = 0;
    rtflag = FALSE;
    defflag = FALSE;
    ptr = fbuf;

    while (ptr < fbufmax)
    {
	switch (*ptr++)
	{
	    case '\n':
		if (rtflag)
		{
		    if (defflag)
		    {
#ifdef DEBUG
			if (DebugLevel > 1)
			    printf("FontInit(): 'n': ENDVEC add\n");
#endif
			CheckFntMemSize();
			FontMem[run++] = ENDVEC;
			defflag = FALSE;
		    }
		}
		else
		    rtflag = TRUE;
		line++;
		break;

	    case ';':
#ifdef DEBUG
		if (DebugLevel > 1)
		    printf("FontInit(): Comment detect\n");
#endif
		rtflag = FALSE;
		while (*ptr++ != '\n');
		line++;
		break;

	    case ' ':
		rtflag = FALSE;
		break;

	    case '*':       // Start of font definition
#ifdef DEBUG
		if (DebugLevel > 1)
		    printf("FontInit(): '*': Run %d Line %d\n",run,line);
#endif
		if (defflag)
		{
#ifdef DEBUG
		    if (DebugLevel > 1)
			printf("FontInit(): '*': ENDVEC add\n");
#endif
		    CheckFntMemSize();
		    FontMem[run++] = ENDVEC;
		}

		code = strtol(ptr,&nptr,10);
		if (nptr == ptr)
		    fnterr("Syntax Error");
		else
		    ptr = nptr;

		if ((code > 255) OR (code < 32))
		    fnterr("ASCII code not in range (32-255)");

		xcord = strtol(ptr,&nptr,10);
		if (nptr == ptr)
		    fnterr("Syntax Error");
		else
		    ptr = nptr;

		ycord = strtol(ptr,&nptr,10);
		if (nptr == ptr)
		    fnterr("Syntax Error");
		else
		    ptr = nptr;
#ifdef DEBUG

		if (DebugLevel > 1)
		    printf("FontInit(): '*': Code: %d AbsSizeX: %d AbsSizeY: %d\n",code,xcord,ycord);
#endif
		if ((xcord < 0) OR (xcord > 127) OR (ycord < 0) OR (ycord > 127))
		    fnterr("Font size not in range (0-127)");

		MyVFChar[code-32].Character = code;
		MyVFChar[code-32].AbsSizeX = xcord;
		MyVFChar[code-32].AbsSizeY = ycord;
		MyVFChar[code-32].VFCKoords = &FontMem[run];

		defflag = TRUE;
		rtflag = FALSE;
		break;

	    default:
#ifdef DEBUG
		if (DebugLevel > 1)
		    printf("FontInit(): coords: Run %d Line %d\n",run,line);
#endif
		rtflag = FALSE;
		if (!defflag)
		    fnterr("Error in Definition");

		ptr--;

		xcord = strtol(ptr,&nptr,10);
		if (nptr == ptr)
		    fnterr("Syntax Error");
		else
		    ptr = nptr;

		ycord = strtol(ptr,&nptr,10);
		if (nptr == ptr)
		    fnterr("Syntax Error");
		else
		    ptr = nptr;

#ifdef DEBUG
		if (DebugLevel > 1)
		    printf("FontInit(): coords: x: %d y: %d\n",xcord,ycord);
#endif
		if (xcord == -2000)
		    fnterr("Error in Definition");

		if (xcord == -1000)
		{
		    CheckFntMemSize();
		    FontMem[run++] = NEWVEC;
		}
		else
		{
		    if ((xcord < -125) OR (xcord > 127) OR (ycord < -125) OR (ycord > 127))
			fnterr("Point position not in range (-125 - 127)");

		    CheckFntMemSize();
		    FontMem[run++] = xcord;
		    FontMem[run++] = ycord;
		}
		break;
	}
    }
    fclose(fh);
}

void RotKoords(short angle,short *x, short *y)
{
    short dummy;

    switch (angle)
    {
	case 0:
	    *y = *y * TextMirror;
	    break;

	case 90:
	    dummy = *x;
	    *x = (-*y) * TextMirror;
	    *y = dummy;
	    break;

	case 180:
	    *x = -*x;
	    *y = -*y * TextMirror;
	    break;

	case 270:
	    dummy = -*x;
	    *x = (*y) * TextMirror;
	    *y = dummy;
	    break;
    }

    *x = (*x)*ActualdpiX/160;
    *y = (*y)*ActualdpiY/160;
}

int DrawVecChar(struct RastPort *Rp, short x, short y, UBYTE z,short size,short angle,short layer)
{
    char  *VecPtr;
    short dx,dy;

    x = Zoll2X(x);
    y = Zoll2Y(y);

    if (layer==INVERS)
	SetDrMd(Rp,JAM1|COMPLEMENT);

    else if (layer == SELECT)
    {
	SetAPen(Rp,SelectCol);
	SetDrMd(Rp,JAM1);
    }
    else
	SetLayerColor(layer);

    if ((ActiveLayer==SOLDERLAYER && (layer==0 || layer==INVERS || layer==SELECT)) || (layer == SOLDERLAYER))
    {
	TextMirror = -1;
	angle += 180;
	if (angle >= 360)
	    angle = angle - 360;
    }
    else
	TextMirror = 1;

    if (MyVFChar[z-32].Character == z)
    {
	if (*(MyVFChar[z-32].VFCKoords) != ENDVEC)
	{
	    VecPtr = MyVFChar[z-32].VFCKoords;
	    dx = (*VecPtr++)*size;
	    dy = (*VecPtr++)*size;
	    RotKoords(angle,&dx,&dy);
	    Move(Rp,x+dx, y-dy);
	    while ((*VecPtr) != ENDVEC)
	    {
		if (*VecPtr == NEWVEC)
		{
		    VecPtr++;
		    dx = (*VecPtr++)*size;
		    dy = (*VecPtr++)*size;  // fr nicht interlaced Screen
		    RotKoords(angle,&dx,&dy);
		    Move(Rp, x+ dx, y-dy);
		}
		else
		{
		    dx = (*VecPtr++)*size;
		    dy = (*VecPtr++)*size;  // fr nicht interlaced Screen
		    RotKoords(angle,&dx,&dy);
		    Draw(Rp, x+dx, y-dy);
		}
	    }
	}


// vier Pixel in y-Richtung oder 8 pixel in x-Richtung entsprechen
// einem 1/20 inch

	switch (angle)
	{
	    case 0:
		len_x = (MyVFChar[z-32].AbsSizeX+3)*8*size;
		len_y = 0;
		break;
	    case 90:
		len_x = 0;
		len_y = -(MyVFChar[z-32].AbsSizeX+3)*8*size;
		break;
	    case 180:
		len_x = -(MyVFChar[z-32].AbsSizeX+3)*8*size;
		len_y = 0;
		break;
	    case 270:
		len_x = 0;
		len_y = (MyVFChar[z-32].AbsSizeX+3)*8*size;
		break;
	}
	SafeSetWriteMask(Rp,255);
	return (TRUE);
    }
    else
    {
	SafeSetWriteMask(Rp,255);
	return (FALSE);
    }
}

int DrawVecTxt(struct RastPort *Rp, short x, short y, UBYTE *txt,short size,short angle,short layer)
{
    short alen_x,alen_y;

    alen_x = alen_y = 0;

    while (*txt != 0)
    {
	DrawVecChar(Rp,x,y, *txt,size,angle,layer);
	x += len_x;
	y += len_y;
	alen_x += len_x;
	alen_y += len_y;
	txt++;
    }
    len_x = alen_x;
    len_y = alen_y;
    return (1);
}

void DrawCursor(short x, short y)
{
    short dx,dy,angle;

    SetDrMd(RP,JAM1|COMPLEMENT);

    x = Zoll2X(x);
    y = Zoll2Y(y);

    dx = 4*TextZoom;
    dy = 8*TextZoom;
    angle = TextAngle;

    if (ActiveLayer == SolderLayer)
    {
	angle += 180;
	if (angle >= 360)
	    angle = angle - 360;
	TextMirror = -1;
    }
    else TextMirror = 1;

    RotKoords(angle,&dx,&dy);

    Move(RP,x,y);
    Draw(RP,x+dx,y);
    Draw(RP,x+dx,y-dy);
    Draw(RP,x,y-dy);
    Draw(RP,x,y);
    WritePixel(RP,x,y);
}

struct Text *AddText(short x_pos, short y_pos, char *text,short size,short angle,short layer)
{
    struct Text *MyText;
    short  len;
    char   *ptr;

    if (strlen(text) == 0)
	return;

    MyText = (struct Text *) PCBEnd;
    len = strlen(text) + 1; // fr NULL-UBYTE
    if (len > 0)
    {
	if ((len % 2) == 1)
	    len += 1;
	if ((char *) MyText + sizeof(struct Text) + len > (char *) PCBMax)
	{
	    rtEZRequest("ERROR: Not enough memory for _AddText","  OK  ",NULL,NULL);
	    return(0);
	}
	UpdateUndo();
	MyText->Type = TEXT_ID;
	MyText->Layer = layer;
	MyText->Size = size;
	MyText->Angle = angle;
	MyText->Textsize = len;
	MyText->x_pos = x_pos;
	MyText->y_pos = y_pos;
	ptr = (char *) PCBEnd + sizeof(struct Text);
	strcpy(ptr,text);
	(char *) PCBEnd += sizeof(struct Text) + len;
	PCBEnd->Type = 0;
	ModCnt++;
	return(PCBEnd);
    }
    return(NULL);
}

void InitDrawTxt(void)
{
    DrawKey = 0;
    x1 = x;
    y1 = y;
    DrawCursor(x1,y1);
}

UBYTE edstr[100];
short edpos;

void SetText(void)
{
    int ch;

    if (DrawKey == 0)
    {
	if (Message->Class == IDCMP_MOUSEMOVE)
	{
	    DrawCursor(x1,y1);
	    x1 = x;
	    y1 = y;
	    DrawCursor(x1,y1);
	}
	else if ((Message->Class==IDCMP_MOUSEBUTTONS) && (Message->Code==SELECTDOWN))
	{
	    DrawKey = 1;
	    edstr[0] = 0;
	    edpos = 0;
	    x2 = x1;
	    y2 = y1;
	}
    }
    else if ((Message->Class == MOUSEBUTTONS) && (Message->Code==SELECTDOWN))
    {
	DrawCursor(x1,y1);
	DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,0);
	x1 = x;
	y1 = y;
	DrawVecTxt(RP,x1,y1,&edstr[0],TextZoom,TextAngle,ActiveLayer);
	x2 = x1;
	y2 = y1;
	x1 += len_x;
	y1 += len_y;
	DrawCursor(x1,y1);
    }

    if (Message->Class == IDCMP_RAWKEY)
    {
	DrawCursor(x1,y1);
	switch (Message->Code)
	{
	    case 0x50:	// F1
		if (TextZoom < 16)
		{
		    if (DrawKey)
		    {
			DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,0);
			TextZoom++;
			DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,ActiveLayer);
			x1 = x2 + len_x;
			y1 = y2 + len_y;
		    } else
			TextZoom++;
		}
		break;
	    case 0x51:	// F2
		if (TextZoom > 1)
		{
		    if (DrawKey)
		    {
			DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,0);
			TextZoom--;
			DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,ActiveLayer);
			x1 = x2 + len_x;
			y1 = y2 + len_y;
		    } else
			TextZoom--;
		}
		break;

	    case 0x52:	// F3
		if (DrawKey)
		{
		    DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,0);
		    TextAngle += 90;
		    if (TextAngle > 270)
			TextAngle = 0;
		    DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,ActiveLayer);
		    x1 = x2 + len_x;
		    y1 = y2 + len_y;
		}
		else TextAngle += 90;
		break;

	    case 0x53:	// F4
		if (DrawKey)
		{
		    DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,0);
		    TextMirror *= -1;
		    DrawVecTxt(RP,x2,y2,&edstr[0],TextZoom,TextAngle,ActiveLayer);
		}
		else TextMirror *= -1;
		break;

	}
	DrawCursor(x1,y1);
	Message->Class = 0;
    }
    if (Message->Class == IDCMP_VANILLAKEY)
    {
	DrawCursor(x1,y1);
	switch (Message->Code)
	{
	    case 8:  // BS
		if (edpos>0 && DrawKey==1)
		{
		    edpos--;
		    DrawVecChar(RP,x1,y1,edstr[edpos],TextZoom,TextAngle,INVERS);
		    DrawVecChar(RP,x1,y1,edstr[edpos],TextZoom,TextAngle,INVERS);
		    // ^^^ dient nur zum Berechnen von len_x & len_y
		    x1 -= len_x;
		    y1 -= len_y;
		    DrawVecChar(RP,x1,y1,edstr[edpos],TextZoom,TextAngle,0);
		    edstr[edpos] = 0;
		}
		break;
	    case 13:
		DrawKey = 0;
		AddText(x2,y2,&edstr[0],TextZoom,TextAngle,ActiveLayer);
		edstr[0] = 0;
		break;

	    default:
		if (DrawKey == 0) break;
		ch = Message->Code;
		if (ch>31 && ch<255)
		{
		    if (edpos < 99 && DrawVecChar(RP,x1,y1,ch,TextZoom,TextAngle,ActiveLayer))
		    {
			edstr[edpos++] = ch;
			edstr[edpos] = 0;
		    }
		    x1 += len_x;
		    y1 += len_y;
		}
		break;
	}
	DrawCursor(x1,y1);
	Message->Class = 0;
    }
}

void DrawDemoText()
{
    DrawVecTxt(RP,10,50,">> CrazyPADS << COPYRIGHT 1993 BY KLAUS ROTTER",2,0,LABELLAYER);
    DrawVecTxt(RP,10,70,"VERSION FOR BETATESTER       DO NOT DISTRIBUTE",2,0,LABELLAYER);
}
